import os
from typing import Union
from prosuite.data_model import TransformedDataset
import prosuite.generated.quality_verification_service_pb2 as service_util
import prosuite.generated.quality_verification_service_pb2_grpc as qa_service
import prosuite.generated.shared_qa_pb2 as shared_qa
import prosuite.generated.shared_types_pb2 as shared_types
import grpc
import logging

from prosuite.quality import Parameter, Specification, XmlSpecification

class EnvelopePerimeter:
    """
    A spatial envelope defined by the bounding coordinates. The spatial reference must match the 
    spatial reference of the datasets.
    """
    def __init__(self, x_min: float, y_min: float, x_max: float,  y_max: float):
        if x_min is None or x_max is None or y_min is None or y_max is None:
            raise Exception('Not all parameters are defined. Please define all envelope parameters')
        self.x_min = x_min
        self.x_max = x_max
        self.y_min = y_min
        self.y_max = y_max


class EsriShapePerimeter:
    """
    A polygon in the Esri shape buffer format.
    """
    def __init__(self, esri_shape: bytes):
        self.esri_shape = esri_shape


class WkbPerimeter:
    """
    A polygon in the OGC well-known-binary format. For example, in ArcPy the geometry's WKB 
    property could be used to acquire this format.
    """
    def __init__(self, wkb: bytes):
        self.wkb = wkb

class VerificationParameters():
    """
    Contains all parameters that can be passed to a verification. 

    :param tile_size: size (in meter) for testing quality conditions
    :type tile_size: int
    :param username: the executing user which will be used in issue features.
    :type username: str
    :param desired_parallel_processing: the desired number of parallel worker processes to be used 
    if the server allows parallel processing.
    :type desired_parallel_processing: int
    """

    def __init__(self, tile_size: int = 5000, user_name: str = None) -> None:
        #:
        self.tile_size: int = tile_size
        #:
        self.user_name: str = user_name
        #:
        self.desired_parallel_processing = 0

class InvolvedTable:
    """
    Represents a table involved in a verification issue.

    :param table_name: The name of the table.
    :type table_name: str
    :param object_ids: A list of object IDs from the table that are involved in the issue.
    :type object_ids: list
    """
    def __init__(self, table_name: str, object_ids: list):
        #:
        self.table_name = table_name
        #:
        self.object_ids = list(object_ids)

class Issue:
    """
    Represents an issue found during the verification.

    To initialize an Issue object, pass the issue message to it.
    :param issue_msg: The issue message.
    :type issue_msg: shared_qa.IssueMsg

    The Issue object extracts some properties from the issue message and makes them available as attributes:
    :ivar description: The description of the issue.
    :ivar involved_objects: A list of InvolvedTable objects that are involved in the issue.
    :ivar geometry: The geometry involved of the issue.
    :ivar issue_code: The issue code ID.
    :ivar allowable: If the issue is allowable.
    :ivar stop_condition: If the issue is a stop condition.
    """
    
    def __init__(self, issue_msg: shared_qa.IssueMsg):
        #:
        self.description = issue_msg.description
        #:
        self.involved_objects = list()
        #:
        self.geometry = issue_msg.issue_geometry
        #:
        self.issue_code = issue_msg.issue_code_id # TODO: test
        #:
        self.allowable = issue_msg.allowable # TODO: test
        #:
        self.stop_condition = issue_msg.stop_condition # TODO: test


        for involved_table_msg in issue_msg.involved_tables:
            involvedTable = InvolvedTable(involved_table_msg.table_name, involved_table_msg.object_ids)
            self.involved_objects.append(involvedTable)


        
class VerificationResponse:
    """
    This class represents a VerificationResponse Message.

    :param message: the actual message
    :param message_level: message level -> see class MessageLevel
    :param service_call_status: service status -> see class ServiceStatus

    The str() method is overridden to return all properties from the VerificationResponse when 
    a VerificationResponse object is printed using pythons print() method. 

    """

    def __init__(self, service_call_status: str, message: str, message_level: str, issue_msgs: list):
        #:
        self.message: str = message
        #:
        self.service_call_status = ServiceStatus.status_0
        #:
        self.message_level = MessageLevel.level_40000
        #:
        self._set_service_call_status(service_call_status)
        #:
        self._set_message_level(message_level)
        #:
        self.issues = list()

        for issue_msg in issue_msgs:
            self.issues.append(Issue(issue_msg))

    def __str__(self):
        return f'service_call_status: {self.service_call_status}\t message_level: {self.message_level} \t ' \
               f'message: {self.message}'

    def _set_service_call_status(self, service_call_status: str):
        if service_call_status == 0:
            self.service_call_status = ServiceStatus.status_0
        if service_call_status == 1:
            self.service_call_status = ServiceStatus.status_1
        if service_call_status == 2:
            self.service_call_status = ServiceStatus.status_2
        if service_call_status == 3:
            self.service_call_status = ServiceStatus.status_3
        if service_call_status == 4:
            self.service_call_status = ServiceStatus.status_4

    def _set_message_level(self, message_level: str):
        if message_level == 110000:
            self.message_level = MessageLevel.level_110000
        if message_level == 70000:
            self.message_level = MessageLevel.level_70000
        if message_level == 60000:
            self.message_level = MessageLevel.level_60000
        if message_level == 40000:
            self.message_level = MessageLevel.level_40000
        if message_level == 30000:
            self.message_level = MessageLevel.level_30000
        if message_level == 10000:
            self.message_level = MessageLevel.level_10000

# TODO: Make _AdvancedParameters 'private'? It should not appear anywhere
class AdvancedParameters():
    def __init__(self, specification, output_dir, perimeter, verification_params: VerificationParameters = None) -> None:

        self.specification:  Union[Specification,
                                   XmlSpecification] = specification
        self.perimeter: Union[EnvelopePerimeter,
                              EsriShapePerimeter, WkbPerimeter] = perimeter
        self.output_dir: str = output_dir
        self.tile_size: int = 5000
        self.user_name: str = None
        self.desired_parallel_processing = 0
        
        if (verification_params):
            self.tile_size = verification_params.tile_size
            self.user_name = verification_params.user_name
            self.desired_parallel_processing = verification_params.desired_parallel_processing


MAX_MESSAGE_LENGTH_MB = 1024

class Service:
    from prosuite.quality import Condition
    from typing import Iterable

    """
    The service class communicates on the http/2 channel with the server and initiates the 
    quality verifications.

    :param host_name: The name or IP address of the host running the quality verification service.
    :type host_name: str
    :param port_nr: The port used by the quality verification service.
    :type port_nr: int
    :param channel_credentials: The channel credentials to be used for TLS/SSL server 
        authentication, if required by the server (Default: None -> No TLS/SSL).

        Use :py:meth:`prosuite.utils.get_ssl_channel_credentials` to create the basic https 
        credentials if the appropria root certificates are in the windows certificate store.
        For advanced scenarios or credentials on a non-windows platform, see the gRPC Python docs
        (https://grpc.github.io/grpc/python/grpc.html).
    :type channel_credentials: grpc.ChannelCredentials  
    """

    ISSUE_GDB = "Issues.gdb"
    """The name of the issue File Geodatabase. 
        It will be written to the output_dir specified in the :py:meth:`prosuite.service.Service.verify` 
        method. This File Geodatabase contains the issues found during the verification and could 
        be used as the source for the Issue Worklist in the ProSuite QA Add-In for ArcGIS Pro.
    """

    XML_REPORT = "verification.xml"
    """The name of the xml verification report.
        It will be written to the output_dir specified in the :py:meth:`prosuite.service.Service.verify` 
        method.
    """

    HTML_REPORT = "verification.html"
    """The name of the html verification report.
        It will be written to the output_dir specified in the :py:meth:`prosuite.service.Service.verify` 
        method.
    """

    def __init__(self, host_name: str, port_nr: int, channel_credentials: grpc.ssl_channel_credentials = None):
        #:
        self.host_name = host_name
        """The host name.

        :meta private:
        """
        #:
        self.port_nr = port_nr
        """The port.

        :meta private:
        """
        #:
        self.ssl_channel_credentials: grpc.ssl_channel_credentials = channel_credentials
        """The TLS/SSL credentials.

        :meta private:
        """

    def verify(self, 
               specification: Union[Specification, XmlSpecification],
               perimeter: Union[EnvelopePerimeter, EsriShapePerimeter, WkbPerimeter] = None, 
               output_dir: str = None, 
               parameters: VerificationParameters = None) -> Iterable[VerificationResponse]:
        """
        Executes a quality verification by running all the quality conditions defined in the 
        provided quality specification.
        Returns a collection of VerificationResponse objects, containing the verification
        messages.

        Please refer to the :ref:`samples <samples-link>` for more details.

        :param specification: The quality specification containing the conditions to be verified.
            It can be either a :py:class:`prosuite.quality.Specification` directly defined in 
            python code or a :py:class:`prosuite.quality.XmlSpecification` from an XML 
            file, for example created by the XML export in the ProSuite Data Dictionary Editor.
        :param perimeter: The perimeter that defines the polygon or extent of the verification.
            Default: None -> Full extent of the verified datasets.
        :param output_dir: The output directory (must be writable / creatable by the service process).
            Default: No output is written by the server process.
        :param parameters: Additional optional verification parameters.
        :return: Iterator for looping over VerificationResponse objects. The verification response
            contains progress information, found issues and, in the final message, the verification results.
        :rtype: Iterator[VerificationResponse]
        """

        advanced_parameters = AdvancedParameters(
            specification, output_dir, perimeter, parameters)

        self._validate_params(advanced_parameters)

        channel = self._create_channel()

        client = qa_service.QualityVerificationGrpcStub(channel)

        request = self._compile_request(advanced_parameters)

        for response_msg in client.VerifyQuality(request):
            yield VerificationResponse(
                message=response_msg.progress.message,
                message_level=response_msg.progress.message_level,
                service_call_status=response_msg.service_call_status,
                issue_msgs=response_msg.issues
            )

    def _create_channel(self):
        message_length = MAX_MESSAGE_LENGTH_MB * 1024 * 1024
        options=[('grpc.max_send_message_length', message_length),
                 ('grpc.max_receive_message_length', message_length)]
        
        if self.ssl_channel_credentials:
            channel = self._create_secure_channel(options)
        else:
            channel = grpc.insecure_channel(f'{self.host_name}:{self.port_nr}', options)
        return channel

    def _validate_params(self, params: AdvancedParameters):
        if params.output_dir is None:
            params.output_dir = ""            
            logging.warn("No output dir is defined")
        if params.specification is None:
            raise Exception(
                "No specification is defined. Please assign verification.specification.")

    def _compile_request(self, parameters: AdvancedParameters):
        req = service_util.VerificationRequest()

        self._configure_verification_parameter_msg(req.parameters, parameters)
        self._configure_specification_msg(req.specification, parameters.specification)

        req.max_parallel_processing = parameters.desired_parallel_processing
        
        return req
    
    def _create_secure_channel(self, options) -> grpc.Channel:
        channel = grpc.secure_channel(
            f'{self.host_name}:{self.port_nr}', self.ssl_channel_credentials, options)
        try:
            grpc.channel_ready_future(channel).result(timeout=5)
            logging.info(
                f'Successfully established secure channel to {self.host_name}')
        except:
            logging.exception(
                f'Timeout. Failed to establish secure channel to {self.host_name}')
        return channel
    
    def _configure_specification_msg(self, 
                                     quality_spec_msg: shared_qa.QualitySpecificationMsg, 
                                     specification: Union[Specification, XmlSpecification]):
        if isinstance(specification, XmlSpecification):
            self._configure_xml_quality_specification_msg(quality_spec_msg.xml_specification, specification)
        else:
            self._configure_condition_list_specification_msg(quality_spec_msg.condition_list_specification, specification)
        
    def _configure_xml_quality_specification_msg(self, 
                                                 xml_specification_msg: shared_qa.XmlQualitySpecificationMsg, 
                                                 specification: XmlSpecification):
        xml_specification_msg.xml = specification.xml_string
        xml_specification_msg.selected_specification_name = specification.specification_name
        if specification.data_source_replacements:
            xml_specification_msg.data_source_replacements.extend(
                specification.data_source_replacements)
        return xml_specification_msg

    def _configure_condition_list_specification_msg(self,
                                                    cond_list_spec_msg: shared_qa.ConditionListSpecificationMsg, specification: Specification):
        cond_list_spec_msg.name = specification.name
        cond_list_spec_msg.description = specification.description
        for condition in specification.get_conditions():
            cond_list_spec_msg.elements.append(
                self._to_xml_quality_specification_element_msg(condition))

        data_sources = self._get_data_sources(specification)

        for key, value in data_sources.items():
            data_source_msg = shared_qa.DataSourceMsg()
            data_source_msg.id = key
            data_source_msg.model_name = key
            data_source_msg.catalog_path = value
            cond_list_spec_msg.data_sources.append(data_source_msg)

        return cond_list_spec_msg

    def _configure_shape_msg(self, 
                             shape_msg: shared_types.ShapeMsg, 
                             perimeter: Union[EnvelopePerimeter, WkbPerimeter, EsriShapePerimeter]):
        if isinstance(perimeter, EnvelopePerimeter):
            perimeter: EnvelopePerimeter
            shape_msg.envelope.x_min = perimeter.x_min
            shape_msg.envelope.x_max = perimeter.x_max
            shape_msg.envelope.y_min = perimeter.y_min
            shape_msg.envelope.y_max = perimeter.y_max
        if isinstance(perimeter, EsriShapePerimeter):
            perimeter: EsriShapePerimeter
            shape_msg.esri_shape = perimeter.esri_shape
        if isinstance(perimeter, WkbPerimeter):
            perimeter: WkbPerimeter
            shape_msg.wkb = bytes(perimeter.wkb)

    def _to_xml_quality_specification_element_msg(self, condition: Condition):
        spec_element = shared_qa.QualitySpecificationElementMsg()
        self._configure_quality_condition_msg(
            condition, spec_element.condition)
        spec_element.allow_errors = condition.allow_errors
        spec_element.category_name = condition.category
        # other props
        return spec_element

    def _configure_verification_parameter_msg(self, 
                                              params_msg: shared_qa.VerificationParametersMsg, 
                                              parameters: AdvancedParameters):
        
        params_msg.tile_size = parameters.tile_size

        if parameters.output_dir != '':
            params_msg.issue_file_gdb_path = os.path.join(
                parameters.output_dir, Service.ISSUE_GDB)
            params_msg.html_report_path = os.path.join(
                parameters.output_dir, Service.HTML_REPORT)
            params_msg.verification_report_path = os.path.join(
                parameters.output_dir, Service.XML_REPORT)
            
        if parameters.perimeter:
            self._configure_shape_msg(
                params_msg.perimeter, parameters.perimeter)


    def _configure_quality_condition_msg(self, 
                                         condition: Condition, 
                                         condition_msg: shared_qa.QualityConditionMsg):
        condition_msg.name = condition.name
        condition_msg.test_descriptor_name = condition.test_descriptor
        condition_msg.description = condition.description
        condition_msg.url = condition.url
        for param in condition.parameters:
            if param.contains_list_of_datasets:
                self._handle_dataset_list(condition_msg, param)
            else:
                condition_msg.parameters.append(self._to_parameter_mgs(param))

    def _handle_dataset_list(self, 
                             condition_msg: shared_qa.QualityConditionMsg, 
                             param: Parameter):
        """
        a Parameter value can be of type Dataset. Or it can be of type list[Dataset].
        if it is a Dataset list, each Dataset in the list should be treated as single Parameter
        """
        if param.contains_list_of_datasets:
            # in this case, param.dataset is actually a list of datasets. we need to unpack the list and create
            # single params for each dataset in the list
            if Parameter.value_is_list_of_datasets(param.dataset):
                for parameter in param.dataset:
                    ds_param = Parameter(param.name, parameter)
                    condition_msg.parameters.append(
                        self._to_parameter_mgs(ds_param))

    @staticmethod
    def _to_parameter_mgs(param: Parameter):
        param_msg = shared_qa.ParameterMsg()
        param_msg.name = param.name

        if isinstance(param.value, TransformedDataset):
            # handle transformed dataset here
            param_msg.where_clause = Service._none_to_emtpy_str(
                param.get_where_clause())
        else:
            param_msg.value = Service._none_to_emtpy_str(
                param.get_string_value())
            param_msg.where_clause = Service._none_to_emtpy_str(
                param.get_where_clause())
            param_msg.workspace_id = param.get_workspace_id()
            param_msg.used_as_reference_data = False

        return param_msg

    @staticmethod
    def _none_to_emtpy_str(value) -> str:
        if value is None:
            return ""
        return value

    def _get_data_sources(self, specification: Specification) -> dict:
        result = {}
        for condition in specification.get_conditions():
            for parameter in condition.parameters:
                if parameter.is_dataset_parameter():
                    if parameter.contains_list_of_datasets:
                        for dataset in parameter.dataset:
                            model = dataset.model
                            result[model.name] = model.catalog_path
                        pass
                    else:
                        model = parameter.dataset.model
                        result[model.name] = model.catalog_path
        return result


class ServiceStatus:
    status_0 = 'Undefined'
    status_1 = 'Running'
    status_2 = 'Cancelled'
    status_3 = 'Finished'
    status_4 = 'Failed'


class MessageLevel:
    level_110000 = 'Fatal'
    level_70000 = 'Error'
    level_60000 = 'Warn'
    level_40000 = "Info"
    level_30000 = 'Debug'
    level_10000 = 'Verbose'


__all__ = ['Condition', 'Parameter', 'Specification']
from enum import Enum
from typing import List
import typing
import prosuite.utils as utils

class Parameter:
    """
    A parameter configures a quality condition. Parameters can represent Datasets (dataset parameter) or scalar values
    (scalar parameters). Parameters have a name and a value.

    Dataset parameters: value is of type dataset. the parameter can retrieve the workspace id (model name) and the where
    clause (filter expression) of the dataset.

    Scalar parameters: value is a simple type (number, string, bool).
    """
    def __init__(self, name: str, value):
        self.contains_list_of_datasets = False
        if self._value_type_is_dataset(value):
            self.dataset = value
        else:
            self.dataset = None

        self.name: str = name
        self.value = value

    def get_string_value(self) -> str:
        if self.dataset:
            if self.contains_list_of_datasets:
                return self.dataset[0].name
            else:
                return self.dataset.name
        else:
            if self.value is None:
                return ""
            else:
                return str(self.value)

    def is_dataset_parameter(self) -> bool:
        if self.dataset:
            return True
        else:
            return False

    def get_workspace_id(self):
        if not self.dataset:
            return ""
        return self.dataset.model.name

    def get_where_clause(self):
        if not self.dataset:
            return ""
        if self.contains_list_of_datasets:
            return self.dataset[0].filter_expression
        else:
            return self.dataset.filter_expression

    @staticmethod
    def value_is_list_of_datasets(value):
        from prosuite.data_model import Dataset
        if isinstance(value, list):
            if all(isinstance(x, Dataset) for x in value):
                return True
        return False

    def _value_type_is_dataset(self, value):
        from prosuite.data_model import BaseDataset
        if Parameter.value_is_list_of_datasets(value):
            self.contains_list_of_datasets = True
            return True
        if isinstance(value, BaseDataset):
            return True
        return False

class IssueFilter():

    def __init__(self, issue_filter_descriptor: str, name: str = "", expression: str = ""):

        #:
        self.name = name
        """
        The unique name of the issue filter.
        """
        #:
        self.issue_filter_descriptor = issue_filter_descriptor
        """
        The issue filter descriptor, i.e. the algorithm used to filter the issues.
        """

        self.parameters: List[Parameter] = []
        """
        The list of parameters. Typically the parameters are specified in the factory method used to create the
        issue filter (see :py:mod:`prosuite.factories.issue_filters`) and do not need to be changed
        through this list.
        """

        self.expression = expression


    def generate_name(self):
        """
        Generates a technical name using the dataset name(s) and the test descriptor. This is the default name of 
        a condition if it was created by the standard factory method from :py:mod:`prosuite.factories.quality_conditions`.
        """
        descriptor = self.issue_filter_descriptor
        params = self.parameters

        self.name = create_name(descriptor, params)


class Condition:
    """
    Defines a quality condition, i.e. the configuration of a test algorithm for one or more datasets. 
    Conditions must be created with the factory methods from :py:mod:`prosuite.factories.quality_conditions` 
    """

    def __init__(self, test_descriptor: str, name: str = ""):

        #:
        self.name = name
        """
        The unique name of the quality condition.
        """
        #:
        self.test_descriptor = test_descriptor
        """
        The test descriptor, i.e. the algorithm used to verify this condition.
        """
        
        # New doc: The type/severity of the quality issues identified by this quality condition. Quality conditions with 
        # “Issue Type = Warning” are considered “soft” conditions, for which exceptions (“Allowed Issues”) may be defined.
        self.allow_errors: bool = IssueType == IssueType.Warning
        # Change by MBR 16/12/23
        #self.allow_errors: bool = self.issue_type == IssueType.Warning
        """
        For internal use only.
        """
        
        # NOTE: Issue type is not processed in the service, because on the server it is called allowed_errors (see above).
        self.issue_type: IssueType = IssueType.Error
        """
        Defines if a failing test returns a warning or an error issue. Quality conditions with 
        Issue Type = Warning are considered “soft” conditions, for which exceptions (“Allowed Issues”) may be defined.
        """

        #:
        self.category: str = ""
        """
        The name of the category, if this issue is assigned to a category.
        """
        #:
        self.stop_on_error: bool = False
        """
        Indicates if the occurrence of an error for an object should stop any further testing of the same object. 
        This can be used to prevent further tests on a feature after a serious geometry error (e.g. incorrectly 
        oriented rings) was detected for the feature.
        The used Test Descriptor provides a standard value for this property. It can optionally be overridden here.
        """
        #:
        self.description: str = ""
        """
        Freely definable description of the Quality Condition. This description can be displayed when viewing 
        issues in the issue navigator, and may contain explanations to the Quality Condition and instructions for 
        correcting the issues.
        """
        #:
        self.url: str = ""
        """
        Optional URL to a website providing additional information for this Quality Condition.
        Certain Quality Conditions require more detailed information about the test logic and/or the correction 
        guidelines than the field “Description” can provide. This information can for example be assembled in a 
        wiki, and the URL may be provided here. When viewing issues in the issue navigator, the corresponding web 
        page can be opened. In the HTML verification reports these URLs are used to render the names of the 
        Quality Conditions as links.
        """
        #:
        self.parameters: List[Parameter] = []
        """
        The list of parameters. Typically the parameters are specified in the factory method used to create the
        quality condition (see :py:mod:`prosuite.factories.quality_conditions`) and do not need to be changed
        through this list.
        """
        #:
        self.issue_filters: List[IssueFilter] = []
        """
        Reserved for future use.
        """
        #:
        self.issue_filter_expression: str
        """
        Reserved for future use.
        """

    def generate_name(self):
        """
        Generates a technical name using the dataset name(s) and the test descriptor. This is the default name of 
        a condition if it was created by the standard factory method from :py:mod:`prosuite.factories.quality_conditions`.
        """
        descriptor = self.test_descriptor
        params = self.parameters

        self.name = create_name(descriptor, params)

# Static method:
# TODO: Extract Parameter class and utils methods such as this one to core.py module
    # to avoid circular dependencies. core.py can then be referencedboth from 
    # data_model (TransformedDataset) and quality (Condition)
def create_name(descriptor, params) -> str:
    first_dataset_parameter = next(
        (p for p in params if p.is_dataset_parameter()), None)
    if first_dataset_parameter:
        ds_param: Parameter = first_dataset_parameter
        if ds_param.contains_list_of_datasets:
            dataset_list: List[str] = [ds.name for ds in ds_param.dataset]
            dataset_names = "_".join(dataset_list)
            
            return f"{descriptor} {dataset_names}"
        else:
            return f"{descriptor} {ds_param.dataset.name}"
            
class Specification:
    def __init__(self, name: str = 'Custom Specification', description: str = ''):
        """
        CustomSpecification stores conditions.
        :param name: specification name
        :type name: str
        :param description: specification description
        :type description: str
        """
        self._conditions: typing.List[Condition] = []
        self.name = name
        self.description = description

    def add_condition(self, condition: Condition):
        """
        Adds conditions to the specification
        """
        self._conditions.append(condition)

    def get_conditions(self) -> typing.List[Condition]:
        """
        Returns the List of conditions
        """
        return self._conditions

class XmlSpecification:
    """
    Represents a specification defined in the xml specification schema:

    :param specification_file: path to the xml specification file
    :type specification_file: str
    :param specification_name: name of the specification (in the xml file) that should be executed. If not defined in the constructor, it needs to be defined later.
    :type specification_name: str
    :param data_source_replacements: a list containing a list with two string elements. these represent a workspace id, and the path to the workspace
    :type data_source_replacements: [[str]] example: [["TLM_Production", "C:/temp/user@topgist.sde"]]
    """

    def __init__(self, specification_file: str, specification_name: str = None,
                 data_source_replacements: List[List[str]] = None):

        self.specification_name: str = None
        self.data_source_replacements = XmlSpecification._parse_datasource_replacements(data_source_replacements)
        self._specification_msg = None
        self.xml_string = XmlSpecification._read_file_as_string(specification_file)
        self.specification_file = specification_file
        self.specification_name = specification_name

    @staticmethod
    def _read_file_as_string(file_path) -> str:
        with open(file_path, "r", encoding="utf-8") as file:
            data = file.read()
            return data

    @staticmethod
    def _parse_datasource_replacements(data_source_replacements: List[List[str]]):
        result = []
        for pair in data_source_replacements:
            result.append(f"{pair[0]}|{pair[1]}")
        return result

    @staticmethod
    def get_specification_names(specification_file: str) -> typing.List[str]:
        """
        Returns a list of the specification names of all specifications in all categories of the xml-specifications doc.

        :param specification_file: path of xml specification file.
        :type specification_file: str
        :return: List of specification names
        :rtype: [str]
        """
        specification_names = []
        xml = utils.objectify_xml(specification_file)
        for cat in xml.Categories:
            for qs in cat.Category.QualitySpecifications:
                name = utils.try_get_lxml_attrib(qs.QualitySpecification, 'name')
                if name:
                    specification_names.append(name)
        return specification_names

class IssueType(Enum): 
    Warning = 0
    Error = 1



import shared_types_pb2 as _shared_types_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class ClassDef(_message.Message):
    __slots__ = ["class_handle", "workspace_handle"]
    CLASS_HANDLE_FIELD_NUMBER: _ClassVar[int]
    WORKSPACE_HANDLE_FIELD_NUMBER: _ClassVar[int]
    class_handle: int
    workspace_handle: int
    def __init__(self, class_handle: _Optional[int] = ..., workspace_handle: _Optional[int] = ...) -> None: ...

class ClassDescriptorMsg(_message.Message):
    __slots__ = ["assembly_name", "type_name"]
    ASSEMBLY_NAME_FIELD_NUMBER: _ClassVar[int]
    TYPE_NAME_FIELD_NUMBER: _ClassVar[int]
    assembly_name: str
    type_name: str
    def __init__(self, type_name: _Optional[str] = ..., assembly_name: _Optional[str] = ...) -> None: ...

class ConditionListSpecificationMsg(_message.Message):
    __slots__ = ["data_sources", "description", "elements", "name"]
    DATA_SOURCES_FIELD_NUMBER: _ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    ELEMENTS_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    data_sources: _containers.RepeatedCompositeFieldContainer[DataSourceMsg]
    description: str
    elements: _containers.RepeatedCompositeFieldContainer[QualitySpecificationElementMsg]
    name: str
    def __init__(self, name: _Optional[str] = ..., description: _Optional[str] = ..., elements: _Optional[_Iterable[_Union[QualitySpecificationElementMsg, _Mapping]]] = ..., data_sources: _Optional[_Iterable[_Union[DataSourceMsg, _Mapping]]] = ...) -> None: ...

class DataRequest(_message.Message):
    __slots__ = ["class_def", "count_only", "rel_query_def", "search_geometry", "sub_fields", "where_clause"]
    CLASS_DEF_FIELD_NUMBER: _ClassVar[int]
    COUNT_ONLY_FIELD_NUMBER: _ClassVar[int]
    REL_QUERY_DEF_FIELD_NUMBER: _ClassVar[int]
    SEARCH_GEOMETRY_FIELD_NUMBER: _ClassVar[int]
    SUB_FIELDS_FIELD_NUMBER: _ClassVar[int]
    WHERE_CLAUSE_FIELD_NUMBER: _ClassVar[int]
    class_def: ClassDef
    count_only: bool
    rel_query_def: RelationshipClassQuery
    search_geometry: _shared_types_pb2.ShapeMsg
    sub_fields: str
    where_clause: str
    def __init__(self, class_def: _Optional[_Union[ClassDef, _Mapping]] = ..., rel_query_def: _Optional[_Union[RelationshipClassQuery, _Mapping]] = ..., where_clause: _Optional[str] = ..., search_geometry: _Optional[_Union[_shared_types_pb2.ShapeMsg, _Mapping]] = ..., sub_fields: _Optional[str] = ..., count_only: bool = ...) -> None: ...

class DataSourceMsg(_message.Message):
    __slots__ = ["catalog_path", "database", "id", "model_name", "schema_owner"]
    CATALOG_PATH_FIELD_NUMBER: _ClassVar[int]
    DATABASE_FIELD_NUMBER: _ClassVar[int]
    ID_FIELD_NUMBER: _ClassVar[int]
    MODEL_NAME_FIELD_NUMBER: _ClassVar[int]
    SCHEMA_OWNER_FIELD_NUMBER: _ClassVar[int]
    catalog_path: str
    database: str
    id: str
    model_name: str
    schema_owner: str
    def __init__(self, id: _Optional[str] = ..., catalog_path: _Optional[str] = ..., model_name: _Optional[str] = ..., database: _Optional[str] = ..., schema_owner: _Optional[str] = ...) -> None: ...

class GdbData(_message.Message):
    __slots__ = ["gdb_object_count", "gdb_objects"]
    GDB_OBJECTS_FIELD_NUMBER: _ClassVar[int]
    GDB_OBJECT_COUNT_FIELD_NUMBER: _ClassVar[int]
    gdb_object_count: int
    gdb_objects: _containers.RepeatedCompositeFieldContainer[_shared_types_pb2.GdbObjectMsg]
    def __init__(self, gdb_object_count: _Optional[int] = ..., gdb_objects: _Optional[_Iterable[_Union[_shared_types_pb2.GdbObjectMsg, _Mapping]]] = ...) -> None: ...

class InstanceConfigurationMsg(_message.Message):
    __slots__ = ["description", "id", "instance_descriptor_name", "name", "parameters", "url"]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    ID_FIELD_NUMBER: _ClassVar[int]
    INSTANCE_DESCRIPTOR_NAME_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    PARAMETERS_FIELD_NUMBER: _ClassVar[int]
    URL_FIELD_NUMBER: _ClassVar[int]
    description: str
    id: int
    instance_descriptor_name: str
    name: str
    parameters: _containers.RepeatedCompositeFieldContainer[ParameterMsg]
    url: str
    def __init__(self, id: _Optional[int] = ..., name: _Optional[str] = ..., instance_descriptor_name: _Optional[str] = ..., url: _Optional[str] = ..., description: _Optional[str] = ..., parameters: _Optional[_Iterable[_Union[ParameterMsg, _Mapping]]] = ...) -> None: ...

class InstanceDescriptorMsg(_message.Message):
    __slots__ = ["class_descriptor", "constructor", "description", "id", "name", "type"]
    CLASS_DESCRIPTOR_FIELD_NUMBER: _ClassVar[int]
    CONSTRUCTOR_FIELD_NUMBER: _ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    ID_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    TYPE_FIELD_NUMBER: _ClassVar[int]
    class_descriptor: ClassDescriptorMsg
    constructor: int
    description: str
    id: int
    name: str
    type: int
    def __init__(self, id: _Optional[int] = ..., name: _Optional[str] = ..., type: _Optional[int] = ..., class_descriptor: _Optional[_Union[ClassDescriptorMsg, _Mapping]] = ..., constructor: _Optional[int] = ..., description: _Optional[str] = ...) -> None: ...

class InvolvedTableMsg(_message.Message):
    __slots__ = ["object_ids", "table_name"]
    OBJECT_IDS_FIELD_NUMBER: _ClassVar[int]
    TABLE_NAME_FIELD_NUMBER: _ClassVar[int]
    object_ids: _containers.RepeatedScalarFieldContainer[int]
    table_name: str
    def __init__(self, table_name: _Optional[str] = ..., object_ids: _Optional[_Iterable[int]] = ...) -> None: ...

class IssueMsg(_message.Message):
    __slots__ = ["affected_component", "allowable", "condition_id", "creation_date_time_ticks", "description", "involved_tables", "issue_code_description", "issue_code_id", "issue_geometry", "legacy_involved_rows", "stop_condition", "test"]
    AFFECTED_COMPONENT_FIELD_NUMBER: _ClassVar[int]
    ALLOWABLE_FIELD_NUMBER: _ClassVar[int]
    CONDITION_ID_FIELD_NUMBER: _ClassVar[int]
    CREATION_DATE_TIME_TICKS_FIELD_NUMBER: _ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    INVOLVED_TABLES_FIELD_NUMBER: _ClassVar[int]
    ISSUE_CODE_DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    ISSUE_CODE_ID_FIELD_NUMBER: _ClassVar[int]
    ISSUE_GEOMETRY_FIELD_NUMBER: _ClassVar[int]
    LEGACY_INVOLVED_ROWS_FIELD_NUMBER: _ClassVar[int]
    STOP_CONDITION_FIELD_NUMBER: _ClassVar[int]
    TEST_FIELD_NUMBER: _ClassVar[int]
    affected_component: str
    allowable: bool
    condition_id: int
    creation_date_time_ticks: int
    description: str
    involved_tables: _containers.RepeatedCompositeFieldContainer[InvolvedTableMsg]
    issue_code_description: str
    issue_code_id: str
    issue_geometry: _shared_types_pb2.ShapeMsg
    legacy_involved_rows: str
    stop_condition: bool
    test: int
    def __init__(self, condition_id: _Optional[int] = ..., issue_geometry: _Optional[_Union[_shared_types_pb2.ShapeMsg, _Mapping]] = ..., description: _Optional[str] = ..., issue_code_id: _Optional[str] = ..., issue_code_description: _Optional[str] = ..., involved_tables: _Optional[_Iterable[_Union[InvolvedTableMsg, _Mapping]]] = ..., legacy_involved_rows: _Optional[str] = ..., allowable: bool = ..., stop_condition: bool = ..., affected_component: _Optional[str] = ..., creation_date_time_ticks: _Optional[int] = ..., test: _Optional[int] = ...) -> None: ...

class LogMsg(_message.Message):
    __slots__ = ["message", "message_level"]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_LEVEL_FIELD_NUMBER: _ClassVar[int]
    message: str
    message_level: int
    def __init__(self, message: _Optional[str] = ..., message_level: _Optional[int] = ...) -> None: ...

class ParameterMsg(_message.Message):
    __slots__ = ["name", "transformer", "used_as_reference_data", "value", "where_clause", "workspace_id"]
    NAME_FIELD_NUMBER: _ClassVar[int]
    TRANSFORMER_FIELD_NUMBER: _ClassVar[int]
    USED_AS_REFERENCE_DATA_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    WHERE_CLAUSE_FIELD_NUMBER: _ClassVar[int]
    WORKSPACE_ID_FIELD_NUMBER: _ClassVar[int]
    name: str
    transformer: InstanceConfigurationMsg
    used_as_reference_data: bool
    value: str
    where_clause: str
    workspace_id: str
    def __init__(self, name: _Optional[str] = ..., value: _Optional[str] = ..., transformer: _Optional[_Union[InstanceConfigurationMsg, _Mapping]] = ..., where_clause: _Optional[str] = ..., workspace_id: _Optional[str] = ..., used_as_reference_data: bool = ...) -> None: ...

class QualityConditionMsg(_message.Message):
    __slots__ = ["condition_id", "condition_issue_filters", "description", "issue_filter_expression", "name", "parameters", "test_descriptor_name", "url"]
    CONDITION_ID_FIELD_NUMBER: _ClassVar[int]
    CONDITION_ISSUE_FILTERS_FIELD_NUMBER: _ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    ISSUE_FILTER_EXPRESSION_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    PARAMETERS_FIELD_NUMBER: _ClassVar[int]
    TEST_DESCRIPTOR_NAME_FIELD_NUMBER: _ClassVar[int]
    URL_FIELD_NUMBER: _ClassVar[int]
    condition_id: int
    condition_issue_filters: _containers.RepeatedCompositeFieldContainer[InstanceConfigurationMsg]
    description: str
    issue_filter_expression: str
    name: str
    parameters: _containers.RepeatedCompositeFieldContainer[ParameterMsg]
    test_descriptor_name: str
    url: str
    def __init__(self, condition_id: _Optional[int] = ..., name: _Optional[str] = ..., test_descriptor_name: _Optional[str] = ..., url: _Optional[str] = ..., description: _Optional[str] = ..., parameters: _Optional[_Iterable[_Union[ParameterMsg, _Mapping]]] = ..., condition_issue_filters: _Optional[_Iterable[_Union[InstanceConfigurationMsg, _Mapping]]] = ..., issue_filter_expression: _Optional[str] = ...) -> None: ...

class QualityConditionVerificationMsg(_message.Message):
    __slots__ = ["error_count", "execute_time", "fulfilled", "quality_condition_id", "row_execute_time", "stop_condition_id", "tile_execute_time"]
    ERROR_COUNT_FIELD_NUMBER: _ClassVar[int]
    EXECUTE_TIME_FIELD_NUMBER: _ClassVar[int]
    FULFILLED_FIELD_NUMBER: _ClassVar[int]
    QUALITY_CONDITION_ID_FIELD_NUMBER: _ClassVar[int]
    ROW_EXECUTE_TIME_FIELD_NUMBER: _ClassVar[int]
    STOP_CONDITION_ID_FIELD_NUMBER: _ClassVar[int]
    TILE_EXECUTE_TIME_FIELD_NUMBER: _ClassVar[int]
    error_count: int
    execute_time: float
    fulfilled: bool
    quality_condition_id: int
    row_execute_time: float
    stop_condition_id: int
    tile_execute_time: float
    def __init__(self, quality_condition_id: _Optional[int] = ..., stop_condition_id: _Optional[int] = ..., fulfilled: bool = ..., error_count: _Optional[int] = ..., execute_time: _Optional[float] = ..., row_execute_time: _Optional[float] = ..., tile_execute_time: _Optional[float] = ...) -> None: ...

class QualitySpecificationElementMsg(_message.Message):
    __slots__ = ["allow_errors", "category_name", "condition", "stop_on_error"]
    ALLOW_ERRORS_FIELD_NUMBER: _ClassVar[int]
    CATEGORY_NAME_FIELD_NUMBER: _ClassVar[int]
    CONDITION_FIELD_NUMBER: _ClassVar[int]
    STOP_ON_ERROR_FIELD_NUMBER: _ClassVar[int]
    allow_errors: bool
    category_name: str
    condition: QualityConditionMsg
    stop_on_error: bool
    def __init__(self, condition: _Optional[_Union[QualityConditionMsg, _Mapping]] = ..., allow_errors: bool = ..., stop_on_error: bool = ..., category_name: _Optional[str] = ...) -> None: ...

class QualitySpecificationMsg(_message.Message):
    __slots__ = ["condition_list_specification", "excluded_condition_ids", "quality_specification_id", "well_known_specification", "xml_specification"]
    CONDITION_LIST_SPECIFICATION_FIELD_NUMBER: _ClassVar[int]
    EXCLUDED_CONDITION_IDS_FIELD_NUMBER: _ClassVar[int]
    QUALITY_SPECIFICATION_ID_FIELD_NUMBER: _ClassVar[int]
    WELL_KNOWN_SPECIFICATION_FIELD_NUMBER: _ClassVar[int]
    XML_SPECIFICATION_FIELD_NUMBER: _ClassVar[int]
    condition_list_specification: ConditionListSpecificationMsg
    excluded_condition_ids: _containers.RepeatedScalarFieldContainer[int]
    quality_specification_id: int
    well_known_specification: int
    xml_specification: XmlQualitySpecificationMsg
    def __init__(self, quality_specification_id: _Optional[int] = ..., well_known_specification: _Optional[int] = ..., xml_specification: _Optional[_Union[XmlQualitySpecificationMsg, _Mapping]] = ..., condition_list_specification: _Optional[_Union[ConditionListSpecificationMsg, _Mapping]] = ..., excluded_condition_ids: _Optional[_Iterable[int]] = ...) -> None: ...

class QualityVerificationDatasetMsg(_message.Message):
    __slots__ = ["dataset_id", "load_time"]
    DATASET_ID_FIELD_NUMBER: _ClassVar[int]
    LOAD_TIME_FIELD_NUMBER: _ClassVar[int]
    dataset_id: int
    load_time: float
    def __init__(self, dataset_id: _Optional[int] = ..., load_time: _Optional[float] = ...) -> None: ...

class QualityVerificationMsg(_message.Message):
    __slots__ = ["cancelled", "condition_verifications", "context_name", "context_type", "end_time_ticks", "fulfilled", "processor_time_seconds", "rows_with_stop_conditions", "saved_verification_id", "specification_description", "specification_id", "specification_name", "start_time_ticks", "user_name", "verification_datasets"]
    CANCELLED_FIELD_NUMBER: _ClassVar[int]
    CONDITION_VERIFICATIONS_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_NAME_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_TYPE_FIELD_NUMBER: _ClassVar[int]
    END_TIME_TICKS_FIELD_NUMBER: _ClassVar[int]
    FULFILLED_FIELD_NUMBER: _ClassVar[int]
    PROCESSOR_TIME_SECONDS_FIELD_NUMBER: _ClassVar[int]
    ROWS_WITH_STOP_CONDITIONS_FIELD_NUMBER: _ClassVar[int]
    SAVED_VERIFICATION_ID_FIELD_NUMBER: _ClassVar[int]
    SPECIFICATION_DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    SPECIFICATION_ID_FIELD_NUMBER: _ClassVar[int]
    SPECIFICATION_NAME_FIELD_NUMBER: _ClassVar[int]
    START_TIME_TICKS_FIELD_NUMBER: _ClassVar[int]
    USER_NAME_FIELD_NUMBER: _ClassVar[int]
    VERIFICATION_DATASETS_FIELD_NUMBER: _ClassVar[int]
    cancelled: bool
    condition_verifications: _containers.RepeatedCompositeFieldContainer[QualityConditionVerificationMsg]
    context_name: str
    context_type: str
    end_time_ticks: int
    fulfilled: bool
    processor_time_seconds: float
    rows_with_stop_conditions: int
    saved_verification_id: int
    specification_description: str
    specification_id: int
    specification_name: str
    start_time_ticks: int
    user_name: str
    verification_datasets: _containers.RepeatedCompositeFieldContainer[QualityVerificationDatasetMsg]
    def __init__(self, saved_verification_id: _Optional[int] = ..., specification_id: _Optional[int] = ..., specification_name: _Optional[str] = ..., specification_description: _Optional[str] = ..., user_name: _Optional[str] = ..., start_time_ticks: _Optional[int] = ..., end_time_ticks: _Optional[int] = ..., fulfilled: bool = ..., cancelled: bool = ..., processor_time_seconds: _Optional[float] = ..., context_type: _Optional[str] = ..., context_name: _Optional[str] = ..., rows_with_stop_conditions: _Optional[int] = ..., condition_verifications: _Optional[_Iterable[_Union[QualityConditionVerificationMsg, _Mapping]]] = ..., verification_datasets: _Optional[_Iterable[_Union[QualityVerificationDatasetMsg, _Mapping]]] = ...) -> None: ...

class RelationshipClassQuery(_message.Message):
    __slots__ = ["join_type", "relationship_class_name", "tables", "where_clause", "workspace_handle"]
    JOIN_TYPE_FIELD_NUMBER: _ClassVar[int]
    RELATIONSHIP_CLASS_NAME_FIELD_NUMBER: _ClassVar[int]
    TABLES_FIELD_NUMBER: _ClassVar[int]
    WHERE_CLAUSE_FIELD_NUMBER: _ClassVar[int]
    WORKSPACE_HANDLE_FIELD_NUMBER: _ClassVar[int]
    join_type: int
    relationship_class_name: str
    tables: _containers.RepeatedScalarFieldContainer[str]
    where_clause: str
    workspace_handle: int
    def __init__(self, relationship_class_name: _Optional[str] = ..., workspace_handle: _Optional[int] = ..., tables: _Optional[_Iterable[str]] = ..., join_type: _Optional[int] = ..., where_clause: _Optional[str] = ...) -> None: ...

class SchemaMsg(_message.Message):
    __slots__ = ["class_definitions", "relclass_definitions"]
    CLASS_DEFINITIONS_FIELD_NUMBER: _ClassVar[int]
    RELCLASS_DEFINITIONS_FIELD_NUMBER: _ClassVar[int]
    class_definitions: _containers.RepeatedCompositeFieldContainer[_shared_types_pb2.ObjectClassMsg]
    relclass_definitions: _containers.RepeatedCompositeFieldContainer[_shared_types_pb2.ObjectClassMsg]
    def __init__(self, class_definitions: _Optional[_Iterable[_Union[_shared_types_pb2.ObjectClassMsg, _Mapping]]] = ..., relclass_definitions: _Optional[_Iterable[_Union[_shared_types_pb2.ObjectClassMsg, _Mapping]]] = ...) -> None: ...

class SchemaRequest(_message.Message):
    __slots__ = ["dataset_ids", "relationship_class_queries"]
    DATASET_IDS_FIELD_NUMBER: _ClassVar[int]
    RELATIONSHIP_CLASS_QUERIES_FIELD_NUMBER: _ClassVar[int]
    dataset_ids: _containers.RepeatedScalarFieldContainer[int]
    relationship_class_queries: _containers.RepeatedCompositeFieldContainer[RelationshipClassQuery]
    def __init__(self, dataset_ids: _Optional[_Iterable[int]] = ..., relationship_class_queries: _Optional[_Iterable[_Union[RelationshipClassQuery, _Mapping]]] = ...) -> None: ...

class VerificationParametersMsg(_message.Message):
    __slots__ = ["filter_table_rows_using_related_geometry", "force_full_scan_for_non_container_tests", "html_report_path", "html_template_path", "invalidate_exceptions_if_any_involved_object_changed", "invalidate_exceptions_if_condition_was_updated", "issue_file_gdb_path", "issue_repository_spatial_reference", "override_allowed_errors", "perimeter", "report_invalid_exceptions", "report_issues_outside_perimeter", "report_unused_exceptions", "save_verification_statistics", "store_related_geometry_for_table_row_issues", "tile_size", "verification_report_path", "write_detailed_verification_report"]
    FILTER_TABLE_ROWS_USING_RELATED_GEOMETRY_FIELD_NUMBER: _ClassVar[int]
    FORCE_FULL_SCAN_FOR_NON_CONTAINER_TESTS_FIELD_NUMBER: _ClassVar[int]
    HTML_REPORT_PATH_FIELD_NUMBER: _ClassVar[int]
    HTML_TEMPLATE_PATH_FIELD_NUMBER: _ClassVar[int]
    INVALIDATE_EXCEPTIONS_IF_ANY_INVOLVED_OBJECT_CHANGED_FIELD_NUMBER: _ClassVar[int]
    INVALIDATE_EXCEPTIONS_IF_CONDITION_WAS_UPDATED_FIELD_NUMBER: _ClassVar[int]
    ISSUE_FILE_GDB_PATH_FIELD_NUMBER: _ClassVar[int]
    ISSUE_REPOSITORY_SPATIAL_REFERENCE_FIELD_NUMBER: _ClassVar[int]
    OVERRIDE_ALLOWED_ERRORS_FIELD_NUMBER: _ClassVar[int]
    PERIMETER_FIELD_NUMBER: _ClassVar[int]
    REPORT_INVALID_EXCEPTIONS_FIELD_NUMBER: _ClassVar[int]
    REPORT_ISSUES_OUTSIDE_PERIMETER_FIELD_NUMBER: _ClassVar[int]
    REPORT_UNUSED_EXCEPTIONS_FIELD_NUMBER: _ClassVar[int]
    SAVE_VERIFICATION_STATISTICS_FIELD_NUMBER: _ClassVar[int]
    STORE_RELATED_GEOMETRY_FOR_TABLE_ROW_ISSUES_FIELD_NUMBER: _ClassVar[int]
    TILE_SIZE_FIELD_NUMBER: _ClassVar[int]
    VERIFICATION_REPORT_PATH_FIELD_NUMBER: _ClassVar[int]
    WRITE_DETAILED_VERIFICATION_REPORT_FIELD_NUMBER: _ClassVar[int]
    filter_table_rows_using_related_geometry: bool
    force_full_scan_for_non_container_tests: bool
    html_report_path: str
    html_template_path: str
    invalidate_exceptions_if_any_involved_object_changed: bool
    invalidate_exceptions_if_condition_was_updated: bool
    issue_file_gdb_path: str
    issue_repository_spatial_reference: _shared_types_pb2.SpatialReferenceMsg
    override_allowed_errors: bool
    perimeter: _shared_types_pb2.ShapeMsg
    report_invalid_exceptions: bool
    report_issues_outside_perimeter: bool
    report_unused_exceptions: bool
    save_verification_statistics: bool
    store_related_geometry_for_table_row_issues: bool
    tile_size: float
    verification_report_path: str
    write_detailed_verification_report: bool
    def __init__(self, tile_size: _Optional[float] = ..., perimeter: _Optional[_Union[_shared_types_pb2.ShapeMsg, _Mapping]] = ..., write_detailed_verification_report: bool = ..., verification_report_path: _Optional[str] = ..., html_report_path: _Optional[str] = ..., html_template_path: _Optional[str] = ..., issue_file_gdb_path: _Optional[str] = ..., store_related_geometry_for_table_row_issues: bool = ..., filter_table_rows_using_related_geometry: bool = ..., override_allowed_errors: bool = ..., report_issues_outside_perimeter: bool = ..., force_full_scan_for_non_container_tests: bool = ..., save_verification_statistics: bool = ..., report_unused_exceptions: bool = ..., report_invalid_exceptions: bool = ..., invalidate_exceptions_if_any_involved_object_changed: bool = ..., invalidate_exceptions_if_condition_was_updated: bool = ..., issue_repository_spatial_reference: _Optional[_Union[_shared_types_pb2.SpatialReferenceMsg, _Mapping]] = ...) -> None: ...

class VerificationProgressMsg(_message.Message):
    __slots__ = ["current_box", "detailed_progress__current_step", "detailed_progress_total_steps", "message", "message_level", "overall_progress_current_step", "overall_progress_total_steps", "processing_step_message", "progress_step", "progress_type", "total_box"]
    CURRENT_BOX_FIELD_NUMBER: _ClassVar[int]
    DETAILED_PROGRESS_TOTAL_STEPS_FIELD_NUMBER: _ClassVar[int]
    DETAILED_PROGRESS__CURRENT_STEP_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_LEVEL_FIELD_NUMBER: _ClassVar[int]
    OVERALL_PROGRESS_CURRENT_STEP_FIELD_NUMBER: _ClassVar[int]
    OVERALL_PROGRESS_TOTAL_STEPS_FIELD_NUMBER: _ClassVar[int]
    PROCESSING_STEP_MESSAGE_FIELD_NUMBER: _ClassVar[int]
    PROGRESS_STEP_FIELD_NUMBER: _ClassVar[int]
    PROGRESS_TYPE_FIELD_NUMBER: _ClassVar[int]
    TOTAL_BOX_FIELD_NUMBER: _ClassVar[int]
    current_box: _shared_types_pb2.EnvelopeMsg
    detailed_progress__current_step: int
    detailed_progress_total_steps: int
    message: str
    message_level: int
    overall_progress_current_step: int
    overall_progress_total_steps: int
    processing_step_message: str
    progress_step: int
    progress_type: int
    total_box: _shared_types_pb2.EnvelopeMsg
    def __init__(self, progress_type: _Optional[int] = ..., progress_step: _Optional[int] = ..., overall_progress_total_steps: _Optional[int] = ..., overall_progress_current_step: _Optional[int] = ..., detailed_progress_total_steps: _Optional[int] = ..., detailed_progress__current_step: _Optional[int] = ..., processing_step_message: _Optional[str] = ..., current_box: _Optional[_Union[_shared_types_pb2.EnvelopeMsg, _Mapping]] = ..., total_box: _Optional[_Union[_shared_types_pb2.EnvelopeMsg, _Mapping]] = ..., message: _Optional[str] = ..., message_level: _Optional[int] = ...) -> None: ...

class WorkContextMsg(_message.Message):
    __slots__ = ["context_name", "context_type", "ddx_id", "type", "verified_dataset_ids", "version_name", "workspace"]
    CONTEXT_NAME_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_TYPE_FIELD_NUMBER: _ClassVar[int]
    DDX_ID_FIELD_NUMBER: _ClassVar[int]
    TYPE_FIELD_NUMBER: _ClassVar[int]
    VERIFIED_DATASET_IDS_FIELD_NUMBER: _ClassVar[int]
    VERSION_NAME_FIELD_NUMBER: _ClassVar[int]
    WORKSPACE_FIELD_NUMBER: _ClassVar[int]
    context_name: str
    context_type: str
    ddx_id: int
    type: int
    verified_dataset_ids: _containers.RepeatedScalarFieldContainer[int]
    version_name: str
    workspace: _shared_types_pb2.WorkspaceMsg
    def __init__(self, ddx_id: _Optional[int] = ..., type: _Optional[int] = ..., workspace: _Optional[_Union[_shared_types_pb2.WorkspaceMsg, _Mapping]] = ..., version_name: _Optional[str] = ..., verified_dataset_ids: _Optional[_Iterable[int]] = ..., context_type: _Optional[str] = ..., context_name: _Optional[str] = ...) -> None: ...

class XmlQualitySpecificationMsg(_message.Message):
    __slots__ = ["data_source_replacements", "selected_specification_name", "xml"]
    DATA_SOURCE_REPLACEMENTS_FIELD_NUMBER: _ClassVar[int]
    SELECTED_SPECIFICATION_NAME_FIELD_NUMBER: _ClassVar[int]
    XML_FIELD_NUMBER: _ClassVar[int]
    data_source_replacements: _containers.RepeatedScalarFieldContainer[str]
    selected_specification_name: str
    xml: str
    def __init__(self, xml: _Optional[str] = ..., selected_specification_name: _Optional[str] = ..., data_source_replacements: _Optional[_Iterable[str]] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: quality_verification_service.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import shared_types_pb2 as shared__types__pb2
import shared_qa_pb2 as shared__qa__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\"quality_verification_service.proto\x12%ProSuite.Microservices.Definitions.QA\x1a\x12shared_types.proto\x1a\x0fshared_qa.proto\"\xa3\x03\n\x13VerificationRequest\x12K\n\x0cwork_context\x18\x01 \x01(\x0b\x32\x35.ProSuite.Microservices.Definitions.QA.WorkContextMsg\x12U\n\rspecification\x18\x02 \x01(\x0b\x32>.ProSuite.Microservices.Definitions.QA.QualitySpecificationMsg\x12T\n\nparameters\x18\x03 \x01(\x0b\x32@.ProSuite.Microservices.Definitions.QA.VerificationParametersMsg\x12I\n\x08\x66\x65\x61tures\x18\x05 \x03(\x0b\x32\x37.ProSuite.Microservices.Definitions.Shared.GdbObjectMsg\x12\x11\n\tuser_name\x18\x06 \x01(\t\x12\x1f\n\x17max_parallel_processing\x18\x07 \x01(\x05\x12\x13\n\x0b\x65nvironment\x18\x08 \x01(\t\"\xe6\x01\n\x17\x44\x61taVerificationRequest\x12K\n\x07request\x18\x01 \x01(\x0b\x32:.ProSuite.Microservices.Definitions.QA.VerificationRequest\x12<\n\x04\x64\x61ta\x18\x02 \x01(\x0b\x32..ProSuite.Microservices.Definitions.QA.GdbData\x12@\n\x06schema\x18\x03 \x01(\x0b\x32\x30.ProSuite.Microservices.Definitions.QA.SchemaMsg\"\x81\x03\n\x1dStandaloneVerificationRequest\x12^\n\x11xml_specification\x18\x01 \x01(\x0b\x32\x41.ProSuite.Microservices.Definitions.QA.XmlQualitySpecificationMsgH\x00\x12l\n\x1c\x63ondition_list_specification\x18\x04 \x01(\x0b\x32\x44.ProSuite.Microservices.Definitions.QA.ConditionListSpecificationMsgH\x00\x12T\n\nparameters\x18\x02 \x01(\x0b\x32@.ProSuite.Microservices.Definitions.QA.VerificationParametersMsg\x12\x18\n\x10output_directory\x18\x03 \x01(\t\x12\x11\n\tuser_name\x18\x06 \x01(\tB\x0f\n\rspecification\"\xca\x03\n\x14VerificationResponse\x12\x1b\n\x13service_call_status\x18\x01 \x01(\x05\x12P\n\x08progress\x18\x02 \x01(\x0b\x32>.ProSuite.Microservices.Definitions.QA.VerificationProgressMsg\x12?\n\x06issues\x18\x03 \x03(\x0b\x32/.ProSuite.Microservices.Definitions.QA.IssueMsg\x12[\n\x14quality_verification\x18\x04 \x01(\x0b\x32=.ProSuite.Microservices.Definitions.QA.QualityVerificationMsg\x12O\n\x12verified_perimeter\x18\x05 \x01(\x0b\x32\x33.ProSuite.Microservices.Definitions.Shared.ShapeMsg\x12T\n\x13obsolete_exceptions\x18\x06 \x03(\x0b\x32\x37.ProSuite.Microservices.Definitions.Shared.GdbObjRefMsg\"\x81\x02\n\x18\x44\x61taVerificationResponse\x12M\n\x08response\x18\x01 \x01(\x0b\x32;.ProSuite.Microservices.Definitions.QA.VerificationResponse\x12H\n\x0c\x64\x61ta_request\x18\x02 \x01(\x0b\x32\x32.ProSuite.Microservices.Definitions.QA.DataRequest\x12L\n\x0eschema_request\x18\x03 \x01(\x0b\x32\x34.ProSuite.Microservices.Definitions.QA.SchemaRequest\"\xbe\x01\n\x1eStandaloneVerificationResponse\x12\x1b\n\x13service_call_status\x18\x01 \x01(\x05\x12>\n\x07message\x18\x02 \x01(\x0b\x32-.ProSuite.Microservices.Definitions.QA.LogMsg\x12?\n\x06issues\x18\x03 \x03(\x0b\x32/.ProSuite.Microservices.Definitions.QA.IssueMsg2\xee\x03\n\x17QualityVerificationGrpc\x12\x8c\x01\n\rVerifyQuality\x12:.ProSuite.Microservices.Definitions.QA.VerificationRequest\x1a;.ProSuite.Microservices.Definitions.QA.VerificationResponse\"\x00\x30\x01\x12\x9a\x01\n\x11VerifyDataQuality\x12>.ProSuite.Microservices.Definitions.QA.DataVerificationRequest\x1a?.ProSuite.Microservices.Definitions.QA.DataVerificationResponse\"\x00(\x01\x30\x01\x12\xa6\x01\n\x13VerifyStandaloneXml\x12\x44.ProSuite.Microservices.Definitions.QA.StandaloneVerificationRequest\x1a\x45.ProSuite.Microservices.Definitions.QA.StandaloneVerificationResponse\"\x00\x30\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'quality_verification_service_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _VERIFICATIONREQUEST._serialized_start=115
  _VERIFICATIONREQUEST._serialized_end=534
  _DATAVERIFICATIONREQUEST._serialized_start=537
  _DATAVERIFICATIONREQUEST._serialized_end=767
  _STANDALONEVERIFICATIONREQUEST._serialized_start=770
  _STANDALONEVERIFICATIONREQUEST._serialized_end=1155
  _VERIFICATIONRESPONSE._serialized_start=1158
  _VERIFICATIONRESPONSE._serialized_end=1616
  _DATAVERIFICATIONRESPONSE._serialized_start=1619
  _DATAVERIFICATIONRESPONSE._serialized_end=1876
  _STANDALONEVERIFICATIONRESPONSE._serialized_start=1879
  _STANDALONEVERIFICATIONRESPONSE._serialized_end=2069
  _QUALITYVERIFICATIONGRPC._serialized_start=2072
  _QUALITYVERIFICATIONGRPC._serialized_end=2566
# @@protoc_insertion_point(module_scope)



import json
import os
import ssl
import typing
from datetime import datetime
import grpc
from lxml import objectify
from typing import Union


def objectify_xml(path_file) -> objectify.ObjectifiedElement:
    """
    Try to objectify the xml file and return the objectified element. Return None if it fails.
    """
    try:
        return objectify.parse(path_file).getroot()
    except:
        raise Exception(f"Error parsing {path_file}")


def try_get_from_oe(prop: str, object: objectify.ObjectifiedElement) -> Union[str, None]:
    try:
        return object[prop]
    except:
        return None


def try_get_lxml_attrib(node: objectify.ObjectifiedElement, attribute_name: str) -> Union[str, None]:
    """
    If the attribute exists on the objectified node then return it, else return None
    """
    if attribute_name in node.attrib.keys():
        return node.attrib[attribute_name]
    else:
        return None


def append_timestamp_to_basepath(base_path):
    now = datetime.now()
    unique_directory = os.path.join(base_path, "{0}{1}{2}_{3}{4}_{5}".format(now.year, now.month, now.day, now.hour,
                                                                             now.minute,
                                                                             now.second))
    return unique_directory


def try_get_from_str_dict(key: str, dictionary: dict, default_value: Union[int, float, str, None] = "") -> \
        Union[int, float, str, None]:
    """
    returns the value of the lookup element if it is available in the dict.
    if the lookup element is not in the dict, the default value is returned.
    """
    if key in dictionary.keys():
        return dictionary[key]
    return default_value


def to_spatial_reference_xml(sr_id: str):
    if sr_id.lower() == "lv95":
        return load_file_as_string("LV95.txt")
    elif sr_id.lower() == "lv03":
        return load_file_as_string("LV_03.txt")
    else:
        raise Exception


def load_json_file(json_file_path: str):
    with open(json_file_path) as json_file:
        return json.load(json_file)


def load_file_as_string(file_name: str) -> str:
    cwd = os.getcwd()
    file_path = os.path.join(os.path.dirname(__file__), file_name)
    with open(file_path, encoding='utf-8') as file:
        lines = file.readlines()
        return "".join(lines)


def to_bool(value: Union[str, bool, int, None] = None, default_value: bool = False) -> bool:
    """"
    treats "true" or "True" or 'yes' or 'Yes' or 1 or '1' as True (bool). Anything else returns False (bool)
    """
    if type(value) == bool:
        return value
    if type(value) == str:
        if str_is_none_or_empty(value):
            return default_value
        if value.upper() == 'TRUE' or value.upper() == 'YES' or value == '1':
            return True
        else:
            return False
    if type(value) == int:
        if value == 1:
            return True
        else:
            return False
    else:
        return default_value


def str_not_empty(string: str) -> bool:
    if string:
        if string == "":
            return False
        else:
            return True
    return False


def str_is_none_or_empty(string: str) -> bool:
    return not str_not_empty(string)


def to_float(value: Union[str, int, float, None], default_value: float = 0) -> Union[float, None]:
    """
    :param value: value that gets converted to float type
    :param default_value: value that is returned in case the float conversion fails
    :return:
    """
    try:
        return float(value)
    except:
        return default_value


def to_int(value: Union[str, int, None], default_value: int = 0) -> Union[int, None]:
    """
    Tries to convert the input value to int. If conversion is not possible, returns the default value.

    :param value: value that gets converted to int type
    :param default_value: value that is returned in case the int conversion fails
    :return: int representation of the input value or 0
   """
    try:
        return int(value)
    except:
        return default_value


def get_value_or_default(value: typing.Any, default: typing.Any) -> typing.Any:
    """
    Returns the value if it is not None. Else returns the default value.
    """
    if value:
        return value
    return default


def get_ssl_channel_credentials() -> grpc.ssl_channel_credentials:
    """
    Creates an ssl channel credentials object for use with an SSL-enabled channel for the 
    authentication with a server that requires TLS/SSL. The appropriate root certificates
    for server authentication are loaded from the windows certificate store.

    :return: A ChannelCredentials object
    """
    ctx = ssl.create_default_context()
    ca_certs = ctx.get_ca_certs(binary_form=True)
    ctx.load_default_certs(purpose=ssl.Purpose.SERVER_AUTH)

    root_certificates_pem = ''

    for cert in ca_certs:
        pem = ssl.DER_cert_to_PEM_cert(cert)
        root_certificates_pem += pem

    return grpc.ssl_channel_credentials(root_certificates=str.encode(root_certificates_pem, 'utf-8'))



__author__ = "The ProSuite Authors"
__copyright__ = "Copyright 2021-2024, The ProSuite Authors"
__license__ = "MIT"
__version__ = "1.3.1.0"
__maintainer__ = "Dira GeoSystems"
__email__ = "programmers@dirageosystems.ch"
__date__  = "30.08.2024"
__status__ = "Production"


from datetime import datetime
from typing import List


# Enums referenced in conditions:

from enum import Enum
class PolylineUsage(Enum):
    AsIs = 0
    AsPolygonIfClosedElseAsPolyline = 1
    AsPolygonIfClosedElseIgnore = 2
    AsPolygonIfClosedElseReportIssue = 3

class NonLinearSegmentType(Enum):
    Bezier = 0
    CircularArc = 1
    EllipticArc = 2

class ShapeAllowed(Enum):
    NoValue = 0
    Cycles = 1
    Branches = 2
    CyclesAndBranches = 3
    InsideBranches = 4
    All = 7

class GroupErrorReporting(Enum):
    ReferToFirstPart = 1
    ShortestGaps = 2
    CombineParts = 4

class GeometryComponent(Enum):
    EntireGeometry = 0
    Boundary = 1
    Vertices = 2
    LineEndPoints = 3
    LineStartPoint = 4
    LineEndPoint = 5
    Centroid = 6
    LabelPoint = 7
    InteriorVertices = 8

class LineFieldValuesConstraint(Enum):
    NoConstraint = 0
    AllEqual = 1
    AllEqualOrValidPointExists = 2
    AtLeastTwoDistinctValuesIfValidPointExists = 3
    UniqueOrValidPointExists = 4

class PointFieldValuesConstraint(Enum):
    NoConstraint = 0
    AllEqualAndMatchAnyLineValue = 1
    AllEqualAndMatchMostFrequentLineValue = 2

class AllowedLineInteriorIntersections(Enum):
    NoValue = 0
    AtVertexOnBothLines = 1

class AllowedEndpointInteriorIntersections(Enum):
    All = 0
    Vertex = 1
    NoValue = 2

class AngleUnit(Enum):
    Radiant = 0
    Degree = 1

class LineMSource(Enum):
    Nearest = 0
    VertexPreferred = 1
    VertexRequired = 2

class MonotonicityDirection(Enum):
    Any = 0
    Increasing = 1
    Decreasing = 2

class InnerRingHandling(Enum):
    NoValue = 0
    IgnoreInnerRings = 1
    IgnoreHorizontalInnerRings = 2

class FieldListType(Enum):
    IgnoredFields = 0
    RelevantFields = 1

class BoundaryLoopErrorGeometry(Enum):
    LoopPolygon = 0
    LoopStartPoint = 1

class BoundaryLoopAreaRelation(Enum):
    IgnoreSmallerOrEqual = 0
    IgnoreLarger = 1

class OrphanErrorType(Enum):
    Both = 1
    OrphanedPoint = 2
    EndPointWithoutPoint = 3

class ExpectedCase(Enum):
    Any = 0
    AllUpper = 1
    AllLower = 2
    Mixed = 3
    NotAllUpper = 4
    NotAllLower = 5

class ExpectedStringDifference(Enum):
    Any = 0
    CaseSensitiveDifference = 1
    CaseInsensitiveDifference = 2

class UniqueStringsConstraint(Enum):
    NoValue = 0
    UniqueExactCase = 1
    UniqueAnyCase = 2

class esriFieldType(Enum):
    esriFieldTypeSmallInteger = 0
    esriFieldTypeInteger = 1
    esriFieldTypeSingle = 2
    esriFieldTypeDouble = 3
    esriFieldTypeString = 4
    esriFieldTypeDate = 5
    esriFieldTypeOID = 6
    esriFieldTypeGeometry = 7
    esriFieldTypeBlob = 8
    esriFieldTypeRaster = 9
    esriFieldTypeGUID = 10
    esriFieldTypeGlobalID = 11
    esriFieldTypeXML = 12
    esriFieldTypeBigInteger = 13

class ConnectionMode(Enum):
    EndpointOnEndpoint = 0
    EndpointOnVertex = 1
    VertexOnVertex = 2

class LineCapStyle(Enum):
    Round = 0
    Butt = 1

class ZComparisonMethod(Enum):
    BoundingBox = 0
    IntersectionPoints = 1

class JoinType(Enum):
    InnerJoin = 1
    LeftJoin = 2
    RightJoin = 3

class ShapeAllowed(Enum):
    NoValue = 0
    Cycles = 1
    Branches = 2
    CyclesAndBranches = 3
    InsideBranches = 4
    All = 7

class GroupErrorReporting(Enum):
    ReferToFirstPart = 1
    ShortestGaps = 2
    CombineParts = 4





# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: shared_types.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12shared_types.proto\x12)ProSuite.Microservices.Definitions.Shared\"\x15\n\x04UUID\x12\r\n\x05value\x18\x01 \x01(\x0c\"I\n\x0b\x45nvelopeMsg\x12\r\n\x05x_min\x18\x01 \x01(\x01\x12\r\n\x05y_min\x18\x02 \x01(\x01\x12\r\n\x05x_max\x18\x03 \x01(\x01\x12\r\n\x05y_max\x18\x04 \x01(\x01\"\xe0\x01\n\x08ShapeMsg\x12\x14\n\nesri_shape\x18\x01 \x01(\x0cH\x00\x12\r\n\x03wkb\x18\x02 \x01(\x0cH\x00\x12J\n\x08\x65nvelope\x18\x03 \x01(\x0b\x32\x36.ProSuite.Microservices.Definitions.Shared.EnvelopeMsgH\x00\x12Y\n\x11spatial_reference\x18\x04 \x01(\x0b\x32>.ProSuite.Microservices.Definitions.Shared.SpatialReferenceMsgB\x08\n\x06\x66ormat\"\xa6\x02\n\x0e\x41ttributeValue\x12\x11\n\x07\x64\x62_null\x18\x01 \x01(\x08H\x00\x12\x19\n\x0fshort_int_value\x18\x02 \x01(\x05H\x00\x12\x18\n\x0elong_int_value\x18\x03 \x01(\x05H\x00\x12\x15\n\x0b\x66loat_value\x18\x04 \x01(\x02H\x00\x12\x16\n\x0c\x64ouble_value\x18\x05 \x01(\x01H\x00\x12\x16\n\x0cstring_value\x18\x06 \x01(\tH\x00\x12\x1f\n\x15\x64\x61te_time_ticks_value\x18\x07 \x01(\x03H\x00\x12\x45\n\nuuid_value\x18\x08 \x01(\x0b\x32/.ProSuite.Microservices.Definitions.Shared.UUIDH\x00\x12\x14\n\nblob_value\x18\t \x01(\x0cH\x00\x42\x07\n\x05value\"\xc6\x01\n\x0cGdbObjectMsg\x12\x11\n\tobject_id\x18\x01 \x01(\x03\x12\x14\n\x0c\x63lass_handle\x18\x02 \x01(\x03\x12I\n\x06values\x18\x03 \x03(\x0b\x32\x39.ProSuite.Microservices.Definitions.Shared.AttributeValue\x12\x42\n\x05Shape\x18\x04 \x01(\x0b\x32\x33.ProSuite.Microservices.Definitions.Shared.ShapeMsg\"7\n\x0cGdbObjRefMsg\x12\x14\n\x0c\x63lass_handle\x18\x01 \x01(\x03\x12\x11\n\tobject_id\x18\x02 \x01(\x03\"\x94\x02\n\x0eObjectClassMsg\x12\x14\n\x0c\x63lass_handle\x18\x01 \x01(\x03\x12\x18\n\x10workspace_handle\x18\x02 \x01(\x03\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\r\n\x05\x61lias\x18\x04 \x01(\t\x12\x15\n\rgeometry_type\x18\x05 \x01(\x05\x12Y\n\x11spatial_reference\x18\x06 \x01(\x0b\x32>.ProSuite.Microservices.Definitions.Shared.SpatialReferenceMsg\x12\x43\n\x06\x66ields\x18\x07 \x03(\x0b\x32\x33.ProSuite.Microservices.Definitions.Shared.FieldMsg\"\xd9\x02\n\x0cWorkspaceMsg\x12\x18\n\x10workspace_handle\x18\x01 \x01(\x03\x12\x19\n\x11workspace_db_type\x18\x02 \x01(\x05\x12\x0c\n\x04path\x18\x03 \x01(\t\x12\x14\n\x0cversion_name\x18\x04 \x01(\t\x12\x1c\n\x14\x64\x65\x66\x61ult_version_name\x18\x05 \x01(\t\x12#\n\x1b\x64\x65\x66\x61ult_version_description\x18\x06 \x01(\t\x12&\n\x1e\x64\x65\x66\x61ult_version_creation_ticks\x18\x07 \x01(\x03\x12Y\n\x15\x63onnection_properties\x18\x08 \x03(\x0b\x32:.ProSuite.Microservices.Definitions.Shared.KeyValuePairMsg\x12*\n\"default_version_modification_ticks\x18\t \x01(\x03\"-\n\x0fKeyValuePairMsg\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"\xab\x01\n\x08\x46ieldMsg\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x12\n\nalias_name\x18\x02 \x01(\t\x12\x0c\n\x04type\x18\x03 \x01(\x05\x12\x0e\n\x06length\x18\x04 \x01(\x05\x12\x13\n\x0b\x64omain_name\x18\x05 \x01(\t\x12\r\n\x05scale\x18\x06 \x01(\x05\x12\x11\n\tprecision\x18\x07 \x01(\x05\x12\x13\n\x0bis_nullable\x18\x08 \x01(\x08\x12\x13\n\x0bis_editable\x18\t \x01(\x08\"\x88\x01\n\x13SpatialReferenceMsg\x12$\n\x1aspatial_reference_esri_xml\x18\x01 \x01(\tH\x00\x12\x1f\n\x15spatial_reference_wkt\x18\x02 \x01(\tH\x00\x12 \n\x16spatial_reference_wkid\x18\x03 \x01(\x05H\x00\x42\x08\n\x06\x66ormat\"\xae\x02\n\x0fResultObjectMsg\x12I\n\x06update\x18\x01 \x01(\x0b\x32\x37.ProSuite.Microservices.Definitions.Shared.GdbObjectMsgH\x00\x12N\n\x06insert\x18\x02 \x01(\x0b\x32<.ProSuite.Microservices.Definitions.Shared.InsertedObjectMsgH\x00\x12I\n\x06\x64\x65lete\x18\x03 \x01(\x0b\x32\x37.ProSuite.Microservices.Definitions.Shared.GdbObjRefMsgH\x00\x12\x15\n\rnotifications\x18\x04 \x03(\t\x12\x13\n\x0bhas_warning\x18\x05 \x01(\x08\x42\t\n\x07\x66\x65\x61ture\"\xba\x01\n\x11InsertedObjectMsg\x12P\n\x0finserted_object\x18\x01 \x01(\x0b\x32\x37.ProSuite.Microservices.Definitions.Shared.GdbObjectMsg\x12S\n\x12original_reference\x18\x02 \x01(\x0b\x32\x37.ProSuite.Microservices.Definitions.Shared.GdbObjRefMsgb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'shared_types_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _UUID._serialized_start=65
  _UUID._serialized_end=86
  _ENVELOPEMSG._serialized_start=88
  _ENVELOPEMSG._serialized_end=161
  _SHAPEMSG._serialized_start=164
  _SHAPEMSG._serialized_end=388
  _ATTRIBUTEVALUE._serialized_start=391
  _ATTRIBUTEVALUE._serialized_end=685
  _GDBOBJECTMSG._serialized_start=688
  _GDBOBJECTMSG._serialized_end=886
  _GDBOBJREFMSG._serialized_start=888
  _GDBOBJREFMSG._serialized_end=943
  _OBJECTCLASSMSG._serialized_start=946
  _OBJECTCLASSMSG._serialized_end=1222
  _WORKSPACEMSG._serialized_start=1225
  _WORKSPACEMSG._serialized_end=1570
  _KEYVALUEPAIRMSG._serialized_start=1572
  _KEYVALUEPAIRMSG._serialized_end=1617
  _FIELDMSG._serialized_start=1620
  _FIELDMSG._serialized_end=1791
  _SPATIALREFERENCEMSG._serialized_start=1794
  _SPATIALREFERENCEMSG._serialized_end=1930
  _RESULTOBJECTMSG._serialized_start=1933
  _RESULTOBJECTMSG._serialized_end=2235
  _INSERTEDOBJECTMSG._serialized_start=2238
  _INSERTEDOBJECTMSG._serialized_end=2424
# @@protoc_insertion_point(module_scope)



__author__ = "The ProSuite Authors"
__copyright__ = "Copyright 2021-2023, The ProSuite Authors"
__license__ = "MIT"
__version__ = "1.2.1.1"
__maintainer__ = "Dira GeoSystems"
__email__ = "programmers@dirageosystems.ch"
__date__  = "22.12.2023"
__status__ = "Production"


from datetime import datetime
from typing import List
from prosuite.quality import Parameter
from prosuite.quality import IssueFilter
from prosuite.data_model import BaseDataset
from prosuite.factories.enums import *

class IssueFilters:
    """
    :noindex:

    Your existing documentation goes here.
    """

    @classmethod
    def not_yet_implemented(cls, feature_class: BaseDataset, tolerance: float) -> IssueFilter:
        """
        Stay tuned
        """
        
        result = IssueFilter("dummy(0)")
        return result


from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class AttributeValue(_message.Message):
    __slots__ = ["blob_value", "date_time_ticks_value", "db_null", "double_value", "float_value", "long_int_value", "short_int_value", "string_value", "uuid_value"]
    BLOB_VALUE_FIELD_NUMBER: _ClassVar[int]
    DATE_TIME_TICKS_VALUE_FIELD_NUMBER: _ClassVar[int]
    DB_NULL_FIELD_NUMBER: _ClassVar[int]
    DOUBLE_VALUE_FIELD_NUMBER: _ClassVar[int]
    FLOAT_VALUE_FIELD_NUMBER: _ClassVar[int]
    LONG_INT_VALUE_FIELD_NUMBER: _ClassVar[int]
    SHORT_INT_VALUE_FIELD_NUMBER: _ClassVar[int]
    STRING_VALUE_FIELD_NUMBER: _ClassVar[int]
    UUID_VALUE_FIELD_NUMBER: _ClassVar[int]
    blob_value: bytes
    date_time_ticks_value: int
    db_null: bool
    double_value: float
    float_value: float
    long_int_value: int
    short_int_value: int
    string_value: str
    uuid_value: UUID
    def __init__(self, db_null: bool = ..., short_int_value: _Optional[int] = ..., long_int_value: _Optional[int] = ..., float_value: _Optional[float] = ..., double_value: _Optional[float] = ..., string_value: _Optional[str] = ..., date_time_ticks_value: _Optional[int] = ..., uuid_value: _Optional[_Union[UUID, _Mapping]] = ..., blob_value: _Optional[bytes] = ...) -> None: ...

class EnvelopeMsg(_message.Message):
    __slots__ = ["x_max", "x_min", "y_max", "y_min"]
    X_MAX_FIELD_NUMBER: _ClassVar[int]
    X_MIN_FIELD_NUMBER: _ClassVar[int]
    Y_MAX_FIELD_NUMBER: _ClassVar[int]
    Y_MIN_FIELD_NUMBER: _ClassVar[int]
    x_max: float
    x_min: float
    y_max: float
    y_min: float
    def __init__(self, x_min: _Optional[float] = ..., y_min: _Optional[float] = ..., x_max: _Optional[float] = ..., y_max: _Optional[float] = ...) -> None: ...

class FieldMsg(_message.Message):
    __slots__ = ["alias_name", "domain_name", "is_editable", "is_nullable", "length", "name", "precision", "scale", "type"]
    ALIAS_NAME_FIELD_NUMBER: _ClassVar[int]
    DOMAIN_NAME_FIELD_NUMBER: _ClassVar[int]
    IS_EDITABLE_FIELD_NUMBER: _ClassVar[int]
    IS_NULLABLE_FIELD_NUMBER: _ClassVar[int]
    LENGTH_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    PRECISION_FIELD_NUMBER: _ClassVar[int]
    SCALE_FIELD_NUMBER: _ClassVar[int]
    TYPE_FIELD_NUMBER: _ClassVar[int]
    alias_name: str
    domain_name: str
    is_editable: bool
    is_nullable: bool
    length: int
    name: str
    precision: int
    scale: int
    type: int
    def __init__(self, name: _Optional[str] = ..., alias_name: _Optional[str] = ..., type: _Optional[int] = ..., length: _Optional[int] = ..., domain_name: _Optional[str] = ..., scale: _Optional[int] = ..., precision: _Optional[int] = ..., is_nullable: bool = ..., is_editable: bool = ...) -> None: ...

class GdbObjRefMsg(_message.Message):
    __slots__ = ["class_handle", "object_id"]
    CLASS_HANDLE_FIELD_NUMBER: _ClassVar[int]
    OBJECT_ID_FIELD_NUMBER: _ClassVar[int]
    class_handle: int
    object_id: int
    def __init__(self, class_handle: _Optional[int] = ..., object_id: _Optional[int] = ...) -> None: ...

class GdbObjectMsg(_message.Message):
    __slots__ = ["Shape", "class_handle", "object_id", "values"]
    CLASS_HANDLE_FIELD_NUMBER: _ClassVar[int]
    OBJECT_ID_FIELD_NUMBER: _ClassVar[int]
    SHAPE_FIELD_NUMBER: _ClassVar[int]
    Shape: ShapeMsg
    VALUES_FIELD_NUMBER: _ClassVar[int]
    class_handle: int
    object_id: int
    values: _containers.RepeatedCompositeFieldContainer[AttributeValue]
    def __init__(self, object_id: _Optional[int] = ..., class_handle: _Optional[int] = ..., values: _Optional[_Iterable[_Union[AttributeValue, _Mapping]]] = ..., Shape: _Optional[_Union[ShapeMsg, _Mapping]] = ...) -> None: ...

class InsertedObjectMsg(_message.Message):
    __slots__ = ["inserted_object", "original_reference"]
    INSERTED_OBJECT_FIELD_NUMBER: _ClassVar[int]
    ORIGINAL_REFERENCE_FIELD_NUMBER: _ClassVar[int]
    inserted_object: GdbObjectMsg
    original_reference: GdbObjRefMsg
    def __init__(self, inserted_object: _Optional[_Union[GdbObjectMsg, _Mapping]] = ..., original_reference: _Optional[_Union[GdbObjRefMsg, _Mapping]] = ...) -> None: ...

class KeyValuePairMsg(_message.Message):
    __slots__ = ["key", "value"]
    KEY_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    key: str
    value: str
    def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...

class ObjectClassMsg(_message.Message):
    __slots__ = ["alias", "class_handle", "fields", "geometry_type", "name", "spatial_reference", "workspace_handle"]
    ALIAS_FIELD_NUMBER: _ClassVar[int]
    CLASS_HANDLE_FIELD_NUMBER: _ClassVar[int]
    FIELDS_FIELD_NUMBER: _ClassVar[int]
    GEOMETRY_TYPE_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    SPATIAL_REFERENCE_FIELD_NUMBER: _ClassVar[int]
    WORKSPACE_HANDLE_FIELD_NUMBER: _ClassVar[int]
    alias: str
    class_handle: int
    fields: _containers.RepeatedCompositeFieldContainer[FieldMsg]
    geometry_type: int
    name: str
    spatial_reference: SpatialReferenceMsg
    workspace_handle: int
    def __init__(self, class_handle: _Optional[int] = ..., workspace_handle: _Optional[int] = ..., name: _Optional[str] = ..., alias: _Optional[str] = ..., geometry_type: _Optional[int] = ..., spatial_reference: _Optional[_Union[SpatialReferenceMsg, _Mapping]] = ..., fields: _Optional[_Iterable[_Union[FieldMsg, _Mapping]]] = ...) -> None: ...

class ResultObjectMsg(_message.Message):
    __slots__ = ["delete", "has_warning", "insert", "notifications", "update"]
    DELETE_FIELD_NUMBER: _ClassVar[int]
    HAS_WARNING_FIELD_NUMBER: _ClassVar[int]
    INSERT_FIELD_NUMBER: _ClassVar[int]
    NOTIFICATIONS_FIELD_NUMBER: _ClassVar[int]
    UPDATE_FIELD_NUMBER: _ClassVar[int]
    delete: GdbObjRefMsg
    has_warning: bool
    insert: InsertedObjectMsg
    notifications: _containers.RepeatedScalarFieldContainer[str]
    update: GdbObjectMsg
    def __init__(self, update: _Optional[_Union[GdbObjectMsg, _Mapping]] = ..., insert: _Optional[_Union[InsertedObjectMsg, _Mapping]] = ..., delete: _Optional[_Union[GdbObjRefMsg, _Mapping]] = ..., notifications: _Optional[_Iterable[str]] = ..., has_warning: bool = ...) -> None: ...

class ShapeMsg(_message.Message):
    __slots__ = ["envelope", "esri_shape", "spatial_reference", "wkb"]
    ENVELOPE_FIELD_NUMBER: _ClassVar[int]
    ESRI_SHAPE_FIELD_NUMBER: _ClassVar[int]
    SPATIAL_REFERENCE_FIELD_NUMBER: _ClassVar[int]
    WKB_FIELD_NUMBER: _ClassVar[int]
    envelope: EnvelopeMsg
    esri_shape: bytes
    spatial_reference: SpatialReferenceMsg
    wkb: bytes
    def __init__(self, esri_shape: _Optional[bytes] = ..., wkb: _Optional[bytes] = ..., envelope: _Optional[_Union[EnvelopeMsg, _Mapping]] = ..., spatial_reference: _Optional[_Union[SpatialReferenceMsg, _Mapping]] = ...) -> None: ...

class SpatialReferenceMsg(_message.Message):
    __slots__ = ["spatial_reference_esri_xml", "spatial_reference_wkid", "spatial_reference_wkt"]
    SPATIAL_REFERENCE_ESRI_XML_FIELD_NUMBER: _ClassVar[int]
    SPATIAL_REFERENCE_WKID_FIELD_NUMBER: _ClassVar[int]
    SPATIAL_REFERENCE_WKT_FIELD_NUMBER: _ClassVar[int]
    spatial_reference_esri_xml: str
    spatial_reference_wkid: int
    spatial_reference_wkt: str
    def __init__(self, spatial_reference_esri_xml: _Optional[str] = ..., spatial_reference_wkt: _Optional[str] = ..., spatial_reference_wkid: _Optional[int] = ...) -> None: ...

class UUID(_message.Message):
    __slots__ = ["value"]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    value: bytes
    def __init__(self, value: _Optional[bytes] = ...) -> None: ...

class WorkspaceMsg(_message.Message):
    __slots__ = ["connection_properties", "default_version_creation_ticks", "default_version_description", "default_version_modification_ticks", "default_version_name", "path", "version_name", "workspace_db_type", "workspace_handle"]
    CONNECTION_PROPERTIES_FIELD_NUMBER: _ClassVar[int]
    DEFAULT_VERSION_CREATION_TICKS_FIELD_NUMBER: _ClassVar[int]
    DEFAULT_VERSION_DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    DEFAULT_VERSION_MODIFICATION_TICKS_FIELD_NUMBER: _ClassVar[int]
    DEFAULT_VERSION_NAME_FIELD_NUMBER: _ClassVar[int]
    PATH_FIELD_NUMBER: _ClassVar[int]
    VERSION_NAME_FIELD_NUMBER: _ClassVar[int]
    WORKSPACE_DB_TYPE_FIELD_NUMBER: _ClassVar[int]
    WORKSPACE_HANDLE_FIELD_NUMBER: _ClassVar[int]
    connection_properties: _containers.RepeatedCompositeFieldContainer[KeyValuePairMsg]
    default_version_creation_ticks: int
    default_version_description: str
    default_version_modification_ticks: int
    default_version_name: str
    path: str
    version_name: str
    workspace_db_type: int
    workspace_handle: int
    def __init__(self, workspace_handle: _Optional[int] = ..., workspace_db_type: _Optional[int] = ..., path: _Optional[str] = ..., version_name: _Optional[str] = ..., default_version_name: _Optional[str] = ..., default_version_description: _Optional[str] = ..., default_version_creation_ticks: _Optional[int] = ..., connection_properties: _Optional[_Iterable[_Union[KeyValuePairMsg, _Mapping]]] = ..., default_version_modification_ticks: _Optional[int] = ...) -> None: ...



"""
prosuite is a ProSuite client API. It supports creation, configuration and execution of QA conditions.
"""
# import generated

import prosuite.factories.enums as Enums
import prosuite.utils as utils
from prosuite.factories.quality_conditions import Conditions
from prosuite.factories.transformers import Transformers
from prosuite.factories.issue_filters import IssueFilters
from prosuite.data_model import Dataset, Model, TransformedDataset
from prosuite.verification import Service
from prosuite.quality import Condition, Parameter, Specification, XmlSpecification
from prosuite.verification import WkbPerimeter, EnvelopePerimeter, EsriShapePerimeter, VerificationParameters, VerificationResponse



class Model:
    """
    The Model represents the data model in a workspace (file-gdb or enterprise-gdb)

    catalog_path examples:
        c:/data.gdb
        c:/enterprise_gdb.sde
    """

    def __init__(self, name, catalog_path):

        #:
        self.name: str = name
        """
        The unique name of the model.
        """
        #:
        self.catalog_path: str = catalog_path
        """
        The catalog path of the associated workspace.
        """

class BaseDataset():
    """
    The base class for datasets representing tabular data. It is either a :class:`.Dataset` or, in the future, a :class:`.TransformedDataset`
    """
    def __init__(self, name: str, filter_expression: str = ""):
        self.name: str = name
        self.filter_expression: str = filter_expression
        pass


class Dataset(BaseDataset):
    """
    A dataset represents data from a table or feature class in a workspace, optionally filtered by an SQL expression.

    :param name: table or featureclass name
    :type name: str
    :param model: The :class:`prosuite.data_model.Model` this dataset belongs to.
    :type model: class:`prosuite.data_model.Model`
    :param filter_expression: A where clause that filters the table. The syntax of the where clause is defined in
        the document SQLSyntax_en.pdf
    :type filter_expression: str, optional
    """

    def __init__(self, name: str, model: Model, filter_expression: str = ""):
        super().__init__(name, filter_expression)
        self.model: Model = model


class TransformedDataset(BaseDataset):

    def __init__(self, transformer_descriptor: str, name: str = "", filter_expression: str = ""):
        from prosuite.quality import Parameter
        from typing import List
        super().__init__(name, filter_expression)

        #:
        self.name = name
        """
        The unique name of the transformed dataset.
        """
        #:
        self.transformer_descriptor = transformer_descriptor
        """
        The transformer descriptor, i.e. the algorithm used to generate this dataset.
        """

        self.parameters: List[Parameter] = []
        """
        The list of parameters. Typically the parameters are specified in the factory method used to create the
        transformed dataset (see :py:mod:`prosuite.factories.transformers`) and do not need to be changed
        through this list.
        """

    def generate_name(self):
        """
        Generates a technical name using the dataset name(s) and the test descriptor. This is the default name of 
        a condition if it was created by the standard factory method from :py:mod:`prosuite.factories.quality_conditions`.
        """
        descriptor = self.transformer_descriptor
        params = self.parameters

        self.name = create_name(descriptor, params)

# Static method (copy from quality condition)
# TODO: Extract Parameter class and utils methods such as this one to core.py module
    # to avoid circular dependencies. core.py can then be referencedboth from 
    # data_model (TransformedDataset) and quality (Condition)
def create_name(descriptor, params) -> str:
    from typing import List
    from prosuite.quality import Parameter

    first_dataset_parameter = next(
        (p for p in params if p.is_dataset_parameter), None)
    if first_dataset_parameter:
        ds_param: Parameter = first_dataset_parameter
        if ds_param.contains_list_of_datasets:
            dataset_list: List[str] = [ds.name for ds in ds_param.dataset]
            dataset_names = "_".join(dataset_list)
            
            return f"{descriptor} {dataset_names}"
        else:
            return f"{descriptor} {ds_param.dataset.name}"


import shared_types_pb2 as _shared_types_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class DetectedIssueMsg(_message.Message):
    __slots__ = ["affected_component", "creation_date_time_ticks", "description", "involved_objects", "issue_code_description", "issue_code_id", "issue_geometry", "stop_condition"]
    AFFECTED_COMPONENT_FIELD_NUMBER: _ClassVar[int]
    CREATION_DATE_TIME_TICKS_FIELD_NUMBER: _ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    INVOLVED_OBJECTS_FIELD_NUMBER: _ClassVar[int]
    ISSUE_CODE_DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    ISSUE_CODE_ID_FIELD_NUMBER: _ClassVar[int]
    ISSUE_GEOMETRY_FIELD_NUMBER: _ClassVar[int]
    STOP_CONDITION_FIELD_NUMBER: _ClassVar[int]
    affected_component: str
    creation_date_time_ticks: int
    description: str
    involved_objects: _containers.RepeatedCompositeFieldContainer[InvolvedObjectsMsg]
    issue_code_description: str
    issue_code_id: str
    issue_geometry: _shared_types_pb2.ShapeMsg
    stop_condition: bool
    def __init__(self, description: _Optional[str] = ..., issue_geometry: _Optional[_Union[_shared_types_pb2.ShapeMsg, _Mapping]] = ..., issue_code_id: _Optional[str] = ..., issue_code_description: _Optional[str] = ..., involved_objects: _Optional[_Iterable[_Union[InvolvedObjectsMsg, _Mapping]]] = ..., stop_condition: bool = ..., affected_component: _Optional[str] = ..., creation_date_time_ticks: _Optional[int] = ...) -> None: ...

class ExecuteTestRequest(_message.Message):
    __slots__ = ["involved_tables", "parameters", "perimeter", "test_name", "workspaces"]
    INVOLVED_TABLES_FIELD_NUMBER: _ClassVar[int]
    PARAMETERS_FIELD_NUMBER: _ClassVar[int]
    PERIMETER_FIELD_NUMBER: _ClassVar[int]
    TEST_NAME_FIELD_NUMBER: _ClassVar[int]
    WORKSPACES_FIELD_NUMBER: _ClassVar[int]
    involved_tables: _containers.RepeatedCompositeFieldContainer[TestDatasetMsg]
    parameters: _containers.RepeatedScalarFieldContainer[str]
    perimeter: _shared_types_pb2.ShapeMsg
    test_name: str
    workspaces: _containers.RepeatedCompositeFieldContainer[_shared_types_pb2.WorkspaceMsg]
    def __init__(self, workspaces: _Optional[_Iterable[_Union[_shared_types_pb2.WorkspaceMsg, _Mapping]]] = ..., involved_tables: _Optional[_Iterable[_Union[TestDatasetMsg, _Mapping]]] = ..., perimeter: _Optional[_Union[_shared_types_pb2.ShapeMsg, _Mapping]] = ..., test_name: _Optional[str] = ..., parameters: _Optional[_Iterable[str]] = ...) -> None: ...

class ExecuteTestResponse(_message.Message):
    __slots__ = ["issues", "progress", "service_call_status"]
    ISSUES_FIELD_NUMBER: _ClassVar[int]
    PROGRESS_FIELD_NUMBER: _ClassVar[int]
    SERVICE_CALL_STATUS_FIELD_NUMBER: _ClassVar[int]
    issues: _containers.RepeatedCompositeFieldContainer[DetectedIssueMsg]
    progress: TestProgressMsg
    service_call_status: int
    def __init__(self, service_call_status: _Optional[int] = ..., progress: _Optional[_Union[TestProgressMsg, _Mapping]] = ..., issues: _Optional[_Iterable[_Union[DetectedIssueMsg, _Mapping]]] = ...) -> None: ...

class InvolvedObjectsMsg(_message.Message):
    __slots__ = ["dataset", "object_ids"]
    DATASET_FIELD_NUMBER: _ClassVar[int]
    OBJECT_IDS_FIELD_NUMBER: _ClassVar[int]
    dataset: _shared_types_pb2.ObjectClassMsg
    object_ids: _containers.RepeatedScalarFieldContainer[int]
    def __init__(self, dataset: _Optional[_Union[_shared_types_pb2.ObjectClassMsg, _Mapping]] = ..., object_ids: _Optional[_Iterable[int]] = ...) -> None: ...

class TestDatasetMsg(_message.Message):
    __slots__ = ["class_definition", "filter_expression"]
    CLASS_DEFINITION_FIELD_NUMBER: _ClassVar[int]
    FILTER_EXPRESSION_FIELD_NUMBER: _ClassVar[int]
    class_definition: _shared_types_pb2.ObjectClassMsg
    filter_expression: str
    def __init__(self, class_definition: _Optional[_Union[_shared_types_pb2.ObjectClassMsg, _Mapping]] = ..., filter_expression: _Optional[str] = ...) -> None: ...

class TestProgressMsg(_message.Message):
    __slots__ = ["message", "progress_current_step", "progress_total_steps"]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    PROGRESS_CURRENT_STEP_FIELD_NUMBER: _ClassVar[int]
    PROGRESS_TOTAL_STEPS_FIELD_NUMBER: _ClassVar[int]
    message: str
    progress_current_step: int
    progress_total_steps: int
    def __init__(self, progress_total_steps: _Optional[int] = ..., progress_current_step: _Optional[int] = ..., message: _Optional[str] = ...) -> None: ...



import shared_types_pb2 as _shared_types_pb2
import shared_qa_pb2 as _shared_qa_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class DataVerificationRequest(_message.Message):
    __slots__ = ["data", "request", "schema"]
    DATA_FIELD_NUMBER: _ClassVar[int]
    REQUEST_FIELD_NUMBER: _ClassVar[int]
    SCHEMA_FIELD_NUMBER: _ClassVar[int]
    data: _shared_qa_pb2.GdbData
    request: VerificationRequest
    schema: _shared_qa_pb2.SchemaMsg
    def __init__(self, request: _Optional[_Union[VerificationRequest, _Mapping]] = ..., data: _Optional[_Union[_shared_qa_pb2.GdbData, _Mapping]] = ..., schema: _Optional[_Union[_shared_qa_pb2.SchemaMsg, _Mapping]] = ...) -> None: ...

class DataVerificationResponse(_message.Message):
    __slots__ = ["data_request", "response", "schema_request"]
    DATA_REQUEST_FIELD_NUMBER: _ClassVar[int]
    RESPONSE_FIELD_NUMBER: _ClassVar[int]
    SCHEMA_REQUEST_FIELD_NUMBER: _ClassVar[int]
    data_request: _shared_qa_pb2.DataRequest
    response: VerificationResponse
    schema_request: _shared_qa_pb2.SchemaRequest
    def __init__(self, response: _Optional[_Union[VerificationResponse, _Mapping]] = ..., data_request: _Optional[_Union[_shared_qa_pb2.DataRequest, _Mapping]] = ..., schema_request: _Optional[_Union[_shared_qa_pb2.SchemaRequest, _Mapping]] = ...) -> None: ...

class StandaloneVerificationRequest(_message.Message):
    __slots__ = ["condition_list_specification", "output_directory", "parameters", "user_name", "xml_specification"]
    CONDITION_LIST_SPECIFICATION_FIELD_NUMBER: _ClassVar[int]
    OUTPUT_DIRECTORY_FIELD_NUMBER: _ClassVar[int]
    PARAMETERS_FIELD_NUMBER: _ClassVar[int]
    USER_NAME_FIELD_NUMBER: _ClassVar[int]
    XML_SPECIFICATION_FIELD_NUMBER: _ClassVar[int]
    condition_list_specification: _shared_qa_pb2.ConditionListSpecificationMsg
    output_directory: str
    parameters: _shared_qa_pb2.VerificationParametersMsg
    user_name: str
    xml_specification: _shared_qa_pb2.XmlQualitySpecificationMsg
    def __init__(self, xml_specification: _Optional[_Union[_shared_qa_pb2.XmlQualitySpecificationMsg, _Mapping]] = ..., condition_list_specification: _Optional[_Union[_shared_qa_pb2.ConditionListSpecificationMsg, _Mapping]] = ..., parameters: _Optional[_Union[_shared_qa_pb2.VerificationParametersMsg, _Mapping]] = ..., output_directory: _Optional[str] = ..., user_name: _Optional[str] = ...) -> None: ...

class StandaloneVerificationResponse(_message.Message):
    __slots__ = ["issues", "message", "service_call_status"]
    ISSUES_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    SERVICE_CALL_STATUS_FIELD_NUMBER: _ClassVar[int]
    issues: _containers.RepeatedCompositeFieldContainer[_shared_qa_pb2.IssueMsg]
    message: _shared_qa_pb2.LogMsg
    service_call_status: int
    def __init__(self, service_call_status: _Optional[int] = ..., message: _Optional[_Union[_shared_qa_pb2.LogMsg, _Mapping]] = ..., issues: _Optional[_Iterable[_Union[_shared_qa_pb2.IssueMsg, _Mapping]]] = ...) -> None: ...

class VerificationRequest(_message.Message):
    __slots__ = ["environment", "features", "max_parallel_processing", "parameters", "specification", "user_name", "work_context"]
    ENVIRONMENT_FIELD_NUMBER: _ClassVar[int]
    FEATURES_FIELD_NUMBER: _ClassVar[int]
    MAX_PARALLEL_PROCESSING_FIELD_NUMBER: _ClassVar[int]
    PARAMETERS_FIELD_NUMBER: _ClassVar[int]
    SPECIFICATION_FIELD_NUMBER: _ClassVar[int]
    USER_NAME_FIELD_NUMBER: _ClassVar[int]
    WORK_CONTEXT_FIELD_NUMBER: _ClassVar[int]
    environment: str
    features: _containers.RepeatedCompositeFieldContainer[_shared_types_pb2.GdbObjectMsg]
    max_parallel_processing: int
    parameters: _shared_qa_pb2.VerificationParametersMsg
    specification: _shared_qa_pb2.QualitySpecificationMsg
    user_name: str
    work_context: _shared_qa_pb2.WorkContextMsg
    def __init__(self, work_context: _Optional[_Union[_shared_qa_pb2.WorkContextMsg, _Mapping]] = ..., specification: _Optional[_Union[_shared_qa_pb2.QualitySpecificationMsg, _Mapping]] = ..., parameters: _Optional[_Union[_shared_qa_pb2.VerificationParametersMsg, _Mapping]] = ..., features: _Optional[_Iterable[_Union[_shared_types_pb2.GdbObjectMsg, _Mapping]]] = ..., user_name: _Optional[str] = ..., max_parallel_processing: _Optional[int] = ..., environment: _Optional[str] = ...) -> None: ...

class VerificationResponse(_message.Message):
    __slots__ = ["issues", "obsolete_exceptions", "progress", "quality_verification", "service_call_status", "verified_perimeter"]
    ISSUES_FIELD_NUMBER: _ClassVar[int]
    OBSOLETE_EXCEPTIONS_FIELD_NUMBER: _ClassVar[int]
    PROGRESS_FIELD_NUMBER: _ClassVar[int]
    QUALITY_VERIFICATION_FIELD_NUMBER: _ClassVar[int]
    SERVICE_CALL_STATUS_FIELD_NUMBER: _ClassVar[int]
    VERIFIED_PERIMETER_FIELD_NUMBER: _ClassVar[int]
    issues: _containers.RepeatedCompositeFieldContainer[_shared_qa_pb2.IssueMsg]
    obsolete_exceptions: _containers.RepeatedCompositeFieldContainer[_shared_types_pb2.GdbObjRefMsg]
    progress: _shared_qa_pb2.VerificationProgressMsg
    quality_verification: _shared_qa_pb2.QualityVerificationMsg
    service_call_status: int
    verified_perimeter: _shared_types_pb2.ShapeMsg
    def __init__(self, service_call_status: _Optional[int] = ..., progress: _Optional[_Union[_shared_qa_pb2.VerificationProgressMsg, _Mapping]] = ..., issues: _Optional[_Iterable[_Union[_shared_qa_pb2.IssueMsg, _Mapping]]] = ..., quality_verification: _Optional[_Union[_shared_qa_pb2.QualityVerificationMsg, _Mapping]] = ..., verified_perimeter: _Optional[_Union[_shared_types_pb2.ShapeMsg, _Mapping]] = ..., obsolete_exceptions: _Optional[_Iterable[_Union[_shared_types_pb2.GdbObjRefMsg, _Mapping]]] = ...) -> None: ...
